local path = "MVP/Main/PreGameAssets/"

if not PreGameMenu then

	-- LEVEL
	local function change_level(level)
		managers.experience:_set_current_level(level)
	end

	local function add_exp(value)
		managers.experience:debug_add_points(value, false)
	end

	-- MONEY
	local function add_money(value)
		managers.money:_add_to_total(value)
	end

	local function reset_money()
		managers.money:reset()
	end

	-- SKILL POINTS
	local function set_skillpoints(value)
		managers.skilltree:_set_points(value)
	end
	
	-- ADD PERK POINTS
	local function set_perkpoints(xp)
		managers.skilltree:give_specialization_points(xp)
	end
	
	-- RESET PERK POINTS
	local function reset_perkpoints()
		managers.skilltree._global.specializations.points = 0
	end
	
	-- UNLOCK ALL PERKS
	local function unlockperks()	
		for _, specialization in ipairs(tweak_data.skilltree.specializations) do
			for _, tree in ipairs(specialization) do
				if (tree.upgrades) then
					for _, upgrade in ipairs(tree.upgrades) do
						managers.upgrades:aquire(upgrade, false)
					end
				end
			end
		end
	end
	
	-- UNLOCK ALL SKILLS
	local function unlock_all_skills()
		set_skillpoints(725)
		
		for tree_id,tree_data in pairs(Global.skilltree_manager.trees) do
			if not tree_data.unlocked then
				managers.skilltree:unlock_tree(tree_id)
			end
			
			for _,skills in pairs(tweak_data.skilltree.trees[tree_id].tiers) do
				for _,skill_id in ipairs(skills) do
					managers.skilltree:unlock(tree_id, skill_id)
				end	
			end	
		end	
	end

	-- Infamy
	local function set_infamy_level(level)
		managers.experience:set_current_rank(level)
	end

	local function set_infamy_points(value)
		managers.infamy:_set_points(value)
	end

	-- Inventory
	local function unlock_slots()
		for i = 1, 300 do
			Global.blackmarket_manager.unlocked_mask_slots[i] = true 
			Global.blackmarket_manager.unlocked_weapon_slots.primaries[i] = true
			Global.blackmarket_manager.unlocked_weapon_slots.secondaries[i] = true
		end	
	end
	
	local unlock_items = function(item_type)
		require(path .. 'unlock_items')
		
		unlock_items(item_type)
	end

	local function delete_items()
		require(path .. 'clear_inventory')
	end

	local clear_slots = function(category)
		require(path .. 'clear_slots')
		
		clear_slots(category)
	end
	
	local function giveitems(argumentTable)
		local itype = argumentTable["type"]
		local times = argumentTable["times"]
		local types = {"weapon_mods", "masks", "materials", "textures", "colors"}
		local skip = {masks = {"character_locked"}, materials = {"plastic"}, colors = {"nothing"}, textures = {"no_color_full_material","no_color_no_material"}}
		if not times then times = 1 end
			if type(itype) == "table" then types = itype end
			if itype == "all" or type(itype) == "table" then
			for i = 1, #types do giveitems({["type"] = types[i], ["times"] = times}) end
			return
		end
		for i=1, times do
			for mat_id,_ in pairs(tweak_data.blackmarket[itype]) do
				if not in_table(skip[itype], mat_id) then
					local global_value = "normal"
					if _.global_value then
						global_value = _.global_value
					elseif _.infamous then
						global_value = "infamous"
					elseif _.dlcs or _.dlc then
						local dlcs = _.dlcs or {}
					if _.dlc then table.insert(dlcs, _.dlc) end
						global_value = dlcs[ math.random(#dlcs) ]
					end
					if _.unlocked == false then _.unlocked = true end
					managers.blackmarket:add_to_inventory(global_value, itype, mat_id, false)
				end	
			end	
		end	
	end
	
	function clearitems(argumentTable)
		local itype = argumentTable["type"]
		local globalval = argumentTable["globalval"]
		local types = {"weapon_mods", "masks", "materials", "textures", "colors"}
		if not globalval then globalval = "all" end
		if type(itype) == "table" then types = itype end
		if itype == "all" or type(itype) == "table" then
			for i = 1, #types do 
				clearitems({["type"] = types[i], ["globalval"] = globalval}) 
			end
		end
		for global_value, categories in pairs(Global.blackmarket_manager.inventory) do
				if (globalval == "all" or globalval == global_value) and categories[itype] then
					for id,amount in pairs(categories[itype]) do
						Global.blackmarket_manager.inventory[global_value][itype][id] = nil
				end 
			end 
		end 
	end
        
	-- Other
	local function unlock_achievements()
		for id in pairs(managers.achievment.achievments) do
			managers.achievment:award(id)
		end	
	end
	
	local function lock_achievements()
		managers.achievment:clear_all_steam()
	end
	
	local function remexclmark()
		Global.blackmarket_manager.new_drops = {}
	end
	
	local function nolimitmod()
		dofile('MVP/Main/PreGameAssets/no_weap_mod_limit.lua')
	end
	
	-- Achievement Table
	local achieveid = {}
	achieveid["armed_and_dangerous"] = 'Armed and Dangerous'
	achieveid["armed_to_the_teeth"] = 'Armed to the Teeth'
	achieveid["armored_1"] = 'Were Gonna Need a Bigger Boat'
	achieveid["armored_10"] = 'Affordable Healthcare'
	achieveid["armored_11"] = 'Heat Around the Corner'
	achieveid["armored_2"] = 'But Wait - Theres More!'
	achieveid["armored_3"] = 'If You Liked It You Should Have Put a Ring on It'
	achieveid["armored_4"] = 'I Do What I Do Best, I Take Scores'
	achieveid["armored_5"] = 'License to Kill'
	achieveid["armored_6"] = 'Let Them Watch'
	achieveid["armored_7"] = 'Im Not a Crook!'
	achieveid["armored_8"] = 'I Did Not Have Sexual Relations With That Bulldozer'
	achieveid["armored_9"] = 'Fool Me Once, Shame on You.'
	achieveid["bigbank_1"] = 'It Takes Two to Tango'
	achieveid["bigbank_10"] = 'Funding Father'
	achieveid["bigbank_2"] = 'It Takes a Pig to Kill a Pig'
	achieveid["bigbank_3"] = 'Sweet Sixteen'
	achieveid["bigbank_4"] = '12 Angry Minutes'
	achieveid["bigbank_5"] = 'Dont Bring the Heat'
	achieveid["bigbank_6"] = 'Backing Bobblehead Bob'
	achieveid["bigbank_7"] = 'Entrapment'
	achieveid["bigbank_8"] = 'You Owe Me One'
	achieveid["bigbank_9"] = 'Dont Forget to Floss'
	achieveid["big_deal"] = 'Big Deal'
	achieveid["big_shot"] = 'Career Criminal'
	achieveid["bilbo_baggin"] = 'They See Me Baggin, They Hatin'
	achieveid["bob_1"] = 'A Vote for Change'
	achieveid["bob_10"] = 'Ghost Riders'
	achieveid["bob_2"] = 'I Invented the Internet'
	achieveid["bob_3"] = 'Im a Swinger'
	achieveid["bob_4"] = 'Storage Hunter'
	achieveid["bob_5"] = 'Master Detective'
	achieveid["bob_6"] = 'Murphys Law'
	achieveid["bob_7"] = 'Death Wish Swinger'
	achieveid["bob_8"] = 'Hot Lava 2.0'
	achieveid["bob_9"] = 'Yes We Can!'
	achieveid["bullet_dodger"] = 'Bullet Dodger'
	achieveid["caribbean_pirate"] = 'Caribbean Pirate'
	achieveid["charliesierra_1"] = 'Sewer Rats'
	achieveid["charliesierra_2"] = 'Is Everything OK?'
	achieveid["charliesierra_3"] = 'Merry Christmas!'
	achieveid["charliesierra_4"] = 'Upside Down'
	achieveid["charliesierra_5"] = 'All Eggs in One Basket'
	achieveid["charliesierra_6"] = 'We Are All Professionals'
	achieveid["charliesierra_7"] = 'Eco Round'
	achieveid["charliesierra_8"] = 'Dead Presents'
	achieveid["charliesierra_9"] = 'Reindeer Games'
	achieveid["death_1"] = 'Wedding Crashers'
	achieveid["death_10"] = 'Making Crime Into an Art Form'
	achieveid["death_11"] = 'Changing the World..'
	achieveid["death_12"] = 'For Better or Worse'
	achieveid["death_13"] = 'Snatch and Grab'
	achieveid["death_14"] = 'Tough As Diamonds'
	achieveid["death_15"] = 'Cash in Before You Cash Out!'
	achieveid["death_16"] = 'Because Thats Where the Money Is'
	achieveid["death_17"] = 'Striking Gold'
	achieveid["death_18"] = 'Cash Is King'
	achieveid["death_19"] = 'Yes Hello, Id Like to Make a De-paws-it'
	achieveid["death_2"] = 'Came in Like a Wrecking Ball'
	achieveid["death_20"] = 'At a Crossroads'
	achieveid["death_21"] = 'Downtown Madness'
	achieveid["death_22"] = 'Shipment and Handling'
	achieveid["death_23"] = 'A Walk in the Park'
	achieveid["death_24"] = 'Its Always Foggy in Washington D.C.'
	achieveid["death_26"] = 'Long-Term Relationship'
	achieveid["death_27"] = 'Tough Act to Follow'
	achieveid["death_28"] = 'Shit Just Got Real'
	achieveid["death_29"] = 'Completely OVERKILL!'
	achieveid["death_3"] = 'Becoming a Regular'
	achieveid["death_30"] = 'OVERKILL Salutes You!'
	achieveid["death_31"] = 'With an Iron Fist'
	achieveid["death_32"] = 'Im a Firestarter'
	achieveid["death_33"] = 'Breaking Bad'
	achieveid["death_34"] = 'Special Delivery'
	achieveid["death_35"] = 'Moonlighting'
	achieveid["death_4"] = 'Blood on the Dance Floor'
	achieveid["death_5"] = 'Like a Fucking Sputnik!'
	achieveid["death_6"] = 'Who Let the Doge Out?'
	achieveid["death_7"] = 'If You Cant Take the Heat...'
	achieveid["death_8"] = 'Into the Lions Den'
	achieveid["death_9"] = 'High Way to Hell'
	achieveid["diamonds_are_forever"] = 'Diamonds Are Forever'
	achieveid["doctor_fantastic"] = 'Doctor Fantastic'
	achieveid["fish_ai"] = 'Fish A.I.'
	achieveid["frappucino_to_go_please"] = 'Cappuccino to Go, Please'
	achieveid["fully_loaded"] = 'Fully Loaded'
	achieveid["f_in_chemistry"] = 'F in Chemistry'
	achieveid["gage2_1"] = 'Doctor Miserable'
	achieveid["gage2_10"] = 'Are You Kidding Me?'
	achieveid["gage2_2"] = 'Cloak and Dagger'
	achieveid["gage2_3"] = 'The Eighth and Final Rule'
	achieveid["gage2_4"] = 'Killins As Easy As Breathing'
	achieveid["gage2_5"] = 'They Drew First Blood, Not Me'
	achieveid["gage2_6"] = 'In Town Youre the Law, Out Here Its Me'
	achieveid["gage2_7"] = 'Dont Push It'
	achieveid["gage2_8"] = 'HOLY SHI-'
	achieveid["gage2_9"] = 'I Aint Got Time to Bleed'
	achieveid["gage3_1"] = 'Build Me an Army Worthy of Crime.net'
	achieveid["gage3_10"] = 'Maximum Penetration'
	achieveid["gage3_11"] = 'Dodge This'
	achieveid["gage3_12"] = 'Surprise Motherfucker'
	achieveid["gage3_13"] = 'Didnt See That Coming Did You?'
	achieveid["gage3_14"] = 'A Taste of Their Own Medicine'
	achieveid["gage3_15"] = 'You Cant Hide'
	achieveid["gage3_16"] = 'Double Kill'
	achieveid["gage3_17"] = 'Public Enemy No. 1'
	achieveid["gage3_2"] = 'The Man With the Golden Gun'
	achieveid["gage3_3"] = 'Lord of the Flies'
	achieveid["gage3_4"] = 'Arachnes Curse'
	achieveid["gage3_5"] = 'Pest Control'
	achieveid["gage3_6"] = 'Seer of Death'
	achieveid["gage3_7"] = 'Far, Far Away'
	achieveid["gage3_8"] = 'Last Action Villain'
	achieveid["gage3_9"] = 'Triple Kill'
	achieveid["gage4_1"] = 'Police Brutality'
	achieveid["gage4_10"] = 'Bang for the Buck'
	achieveid["gage4_11"] = 'No Heist for Old Men'
	achieveid["gage4_12"] = 'Four Monkeys'
	achieveid["gage4_2"] = 'Lock, Stock & Eight Smoking Barrels'
	achieveid["gage4_3"] = 'Swing Dancing'
	achieveid["gage4_4"] = 'Seven Eleven'
	achieveid["gage4_5"] = 'Shotgun 101'
	achieveid["gage4_6"] = 'Knock, Knock'
	achieveid["gage4_7"] = 'Everyday Im Shovelin'
	achieveid["gage4_8"] = 'Clay Pigeon Shooting'
	achieveid["gage4_9"] = 'Shock and Awe'
	achieveid["gage5_1"] = 'Precision Aiming'
	achieveid["gage5_10"] = 'Tour de Clarion'
	achieveid["gage5_2"] = 'Big Bada Boom'
	achieveid["gage5_3"] = 'Army of One'
	achieveid["gage5_4"] = 'So Many Choices'
	achieveid["gage5_5"] = 'Artillery Barrage'
	achieveid["gage5_6"] = 'Unusual Suspects'
	achieveid["gage5_7"] = 'Not Today'
	achieveid["gage5_8"] = 'Hammertime'
	achieveid["gage5_9"] = 'Rabbit Hunting'
	achieveid["gage_1"] = 'Wanted'
	achieveid["gage_10"] = 'Share the Love'
	achieveid["gage_2"] = '3000 Miles to the Safe House'
	achieveid["gage_3"] = 'Commando'
	achieveid["gage_4"] = 'Public Enemies'
	achieveid["gage_5"] = 'Inception'
	achieveid["gage_6"] = 'Hard Corps'
	achieveid["gage_7"] = 'Above the Law'
	achieveid["gage_8"] = 'Guns Are Like Shoes'
	achieveid["gage_9"] = 'Fire in the Hole!'
	achieveid["gmod_1"] = 'Praying Mantis'
	achieveid["gmod_10"] = 'Russian Operator'
	achieveid["gmod_2"] = 'Bulls-Eye'
	achieveid["gmod_3"] = 'My Spider Sense is Tingling'
	achieveid["gmod_4"] = 'Eagle Eyes'
	achieveid["gmod_5"] = 'Like A Boy Killing Snakes'
	achieveid["gmod_6"] = 'There and Back Again'
	achieveid["gmod_7"] = 'High Speed, Low Drag'
	achieveid["gmod_8"] = 'Point n Shoot'
	achieveid["gmod_9"] = 'Mall Ninja'
	achieveid["going_all_in"] = 'Im Going All-in!'
	achieveid["going_places"] = 'Going Places'
	achieveid["gone_in_30_seconds"] = 'Smooth Criminal'
	achieveid["guilty_of_crime"] = 'Guilty of Crime'
	achieveid["halloween_1"] = 'I Am the One Who Knocks'
	achieveid["halloween_10"] = 'From Russia With Love'
	achieveid["halloween_2"] = 'Full Measure'
	achieveid["halloween_3"] = 'Afraid of the Dark'
	achieveid["halloween_4"] = 'Witch Doctor'
	achieveid["halloween_5"] = 'Its Alive! ITS ALIVE!'
	achieveid["halloween_6"] = 'Pump-Action'
	achieveid["halloween_7"] = 'No One Can Hear You Scream'
	achieveid["halloween_8"] = 'The Pumpkin King Made Me Do It!'
	achieveid["halloween_9"] = 'Christmas Came Early'
	achieveid["halloween_nightmare_1"] = 'First Nightmare'
	achieveid["halloween_nightmare_2"] = 'Second Nightmare'
	achieveid["halloween_nightmare_3"] = 'Third Nightmare'
	achieveid["halloween_nightmare_4"] = 'Fourth Nightmare'
	achieveid["hot_wheels"] = 'Coming in Hot'
	achieveid["how_do_you_like_me_now"] = 'How Do You Like Me Now?'
	achieveid["ignominy_1"] = 'Becoming Infamous'
	achieveid["ignominy_2"] = 'Becoming Disgraceful'
	achieveid["ignominy_3"] = 'Becoming Wicked'
	achieveid["ignominy_4"] = 'Becoming Notorious'
	achieveid["ignominy_5"] = 'Becoming Vicious'
	achieveid["im_a_healer_tank_damage_dealer"] = 'Im a Healer-Tank-Damage-Dealer'
	achieveid["iron_man"] = 'Man of Iron'
	achieveid["i_knew_what_i_was_doing_was_wrong"] = 'I Knew What I Did Was Wrong'
	achieveid["i_wasnt_even_there"] = 'I Wasnt Even There!'
	achieveid["king_of_the_hill"] = 'King of the Hill'
	achieveid["kosugi_1"] = 'I Will Fade to Dark'
	achieveid["kosugi_2"] = 'I Will Pass Through Walls'
	achieveid["kosugi_3"] = 'I Will Take With Impunity'
	achieveid["kosugi_4"] = 'I Will Walk Faceless Among Men'
	achieveid["kosugi_5"] = 'I Am Ninja'
	achieveid["kosugi_6"] = 'I Will Die and Die and Die Again'
	achieveid["lets_do_this"] = 'Lets Do Th...'
	achieveid["like_an_angry_bear"] = 'Like an Angry Bear'
	achieveid["lord_of_war"] = 'Lord of War'
	achieveid["masked_villain"] = 'Masked Villain'
	achieveid["most_wanted"] = 'Most Wanted'
	achieveid["murphys_laws"] = 'I Got It, I Got It!'
	achieveid["no_one_cared_who_i_was"] = 'No One Cared Who I Was...'
	achieveid["no_turning_back"] = 'No Turning Back'
	achieveid["pig_1"] = 'Wrong Number'
	achieveid["pig_2"] = 'Walk Faster'
	achieveid["pig_3"] = 'Do You Like Hurting Other People?'
	achieveid["pig_4"] = 'Overdose'
	achieveid["pig_5"] = 'Sounds of Animals Fighting'
	achieveid["pig_6"] = '+1 (786) 519-3708'
	achieveid["pig_7"] = 'Phew!'
	achieveid["pink_panther"] = 'Painting Yourself Into a Corner'
	achieveid["short_fuse"] = 'Short Fuse'
	achieveid["spend_money_to_make_money"] = 'Spend Money to Make Money'
	achieveid["the_first_line"] = 'The First Line'
	achieveid["the_wire"] = 'Guessing Game'
	achieveid["tip_the_scales"] = 'Tip the Scales'
	achieveid["weapon_collector"] = 'Weapon Collector'
	achieveid["window_cleaner"] = 'Shoot the Glass!'
	achieveid["would_you_like_your_receipt"] = 'Would You Like Your Receipt?'
	achieveid["yeah_hes_a_gold_digger"] = 'Yeah Hes a Gold Digger'
	achieveid["you_gotta_start_somewhere"] = 'You Gotta Start Somewhere'
	achieveid["you_shall_not_pass"] = 'You Shall Not Pass!'
	achieveid["bulldog_1"] = 'Why Dont We Just Use A Spoon?'
	achieveid["bulldog_2"] = 'No Bars Can Hold Me'
	achieveid["bulldog_3"] = 'Havoc in the Streets'
	achieveid["bulldog_4"] = 'Cavity'
	achieveid["deer_1"] = 'Vlads Little Helpers'
	achieveid["deer_2"] = 'Stealing Christmas'
	achieveid["deer_3"] = 'Cancelling Santas Christmas'
	achieveid["deer_4"] = 'Claustrophobia'
	achieveid["deer_5"] = 'Like Turkeys Voting for an Early Christmas'
	achieveid["deer_6"] = 'Riders On the Snowstorm'
	achieveid["deer_7"] = "Impossible, It Can't Be. Is It?"
	achieveid["eagle_1"] = "Death From Below"
	achieveid["eagle_2"] = "Special Operations Executions"
	achieveid["eagle_3"] = "Wind Of Change"
	achieveid["eagle_4"] = "So Uncivilized"
	achieveid["eagle_5"] = "Bullet Hell"
	achieveid["bat_2"] = "Cat Burglar"
	achieveid["bat_3"] = "Culture Vultures"
	achieveid["bat_4"] = "Smoke And Mirrors"
	achieveid["bat_1"] = 'Devil of a Job'
	achieveid["bat_5"] = 'Honor Among Thieves'
	achieveid["bat_6"] = 'Diamonds in the Rough'
	achieveid["frog_1"] = 'Tabula Rasa'
	achieveid["gorilla_1"] = 'Knockout'
	achieveid["squek"] = 'State of the Art'
	achieveid["cow_1"] = 'Derailed'
	achieveid["cow_2"] = 'A pine in the Ass'
	achieveid["cow_3"] = 'Beaver Team'
	achieveid["cow_4"] = 'Pump It Up'
	achieveid["cow_5"] = 'Opressor'
	achieveid["cow_6"] = 'Maiden Voyage'
	achieveid["cow_7"] = "Fisherman's Friend"
	achieveid["cow_8"] = 'Sneaking With The Fishes'
	achieveid["cow_9"] = 'Breaking Dead'
	achieveid["cow_10"] = "I've Got The Power"
	achieveid["cow_11"] = 'Done In 60 Seconds'

	function remachieve(achieveid)
		if managers.achievment then
			for id,_ in pairs(managers.achievment.achievments) do
				if managers.achievment:get_info(achieveid).awarded then
					managers.achievment:clear_steam(achieveid)
				end	
			end	
		end	
	end
				
	function addachieve(achieveid)
		if managers.achievment then
			for id,_ in pairs(managers.achievment.achievments) do
				managers.achievment:award(achieveid)
			end
		end	
	end
---------------------------------------------------------------------------------------------------------------------------------
	PreGameMenu = CustomMenuClass:new()
	PreGameMenu:addMainMenu('main_menu', {title = 'Pre-Game Menu - Select Menu', maxRows = 11})
	PreGameMenu:addMenu('infamy_menu', {title = 'Infamy Menu - Set Your Infamy'})
	PreGameMenu:addMenu('inventory_menu', {title = 'Inventory Menu - Select Option', maxRows = 12})
	PreGameMenu:addMenu('clear_inventory_menu', {title = 'Clear Inventory Menu - Select Option'})
	PreGameMenu:addMenu('add_inventory_menu', {title = 'Add To Inventory Menu - Select Option'})
	PreGameMenu:addMenu('level_menu', {title = 'Level Menu - Select Your Level', maxRows = 11})
	PreGameMenu:addMenu('money_menu', {title = 'Money Menu - Select Option'})
	PreGameMenu:addMenu('skill_menu', {title = 'Skill And Perk Menu - Set Your Skill'})
	PreGameMenu:addMenu('add_achieve_menu', {title = 'Add Achievement Menu - Select Achievement' , maxColumns = 2, maxRows = 11})
	PreGameMenu:addMenu('rem_achieve_menu', {title = 'Remove Achievement Menu - Select Achievement', maxColumns = 2, maxRows = 11})
	
	PreGameMenu:addInformationOption('main_menu', 'MENUS', {textColor = Color.red})
	PreGameMenu:addMenuOption('main_menu', 'Infamy Menu', 'infamy_menu', {rectHighlightColor = Color.red})
	-- Infamy Menu Column 1
	PreGameMenu:addInformationOption('infamy_menu', 'INFAMY OPTION', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('infamy_menu', 'Reset Infamy', {callback = function() set_infamy_level(0) end})
	PreGameMenu:addGap('infamy_menu')
	PreGameMenu:addInformationOption('infamy_menu', 'INFAMY LEVEL OPTIONS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Level To 1', {callback = set_infamy_level, callbackData = 1, closeMenu = true})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Level To 2', {callback = set_infamy_level, callbackData = 2, closeMenu = true})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Level To 3', {callback = set_infamy_level, callbackData = 3, closeMenu = true})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Level To 4', {callback = set_infamy_level, callbackData = 4, closeMenu = true})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Level To 5', {callback = set_infamy_level, callbackData = 5, closeMenu = true})
	PreGameMenu:addGap('infamy_menu')
	PreGameMenu:addGap('infamy_menu')
	-- Infamy Menu Column 2
	PreGameMenu:addGap('infamy_menu')
	PreGameMenu:addGap('infamy_menu')
	PreGameMenu:addInformationOption('infamy_menu', 'INFAMY POINTS OPTIONS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Points To 1', {callback = function() set_infamy_points(1) end, closeMenu = true})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Points To 2', {callback = function() set_infamy_points(2) end, closeMenu = true})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Points To 3', {callback = function() set_infamy_points(3) end, closeMenu = true})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Points To 4', {callback = function() set_infamy_points(4) end, closeMenu = true})
	PreGameMenu:addOption('infamy_menu', 'Set Infamy Points To 5', {callback = function() set_infamy_points(5) end, closeMenu = true})
	
	PreGameMenu:addMenuOption('main_menu', 'Inventory Menu', 'inventory_menu', {rectHighlightColor = Color.red})
	-- Inventory Menu Column 1
	PreGameMenu:addInformationOption('inventory_menu', 'INVENTORY MENUS', {textColor = Color.red})
	PreGameMenu:addMenuOption('inventory_menu', 'Add To Inventory Menu', 'add_inventory_menu', {rectHighlightColor = Color.red})
	PreGameMenu:addMenuOption('inventory_menu', 'Clear Inventory Menu', 'clear_inventory_menu', {rectHighlightColor = Color.red})
	PreGameMenu:addGap('inventory_menu')
	PreGameMenu:addInformationOption('inventory_menu', 'UNLOCK OPTIONS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('inventory_menu', 'Unlock All Items', {callback = unlock_items, callbackData = "all"})
	PreGameMenu:addOption('inventory_menu', 'Unlock All Slots', {callback = unlock_slots})
	PreGameMenu:addOption('inventory_menu', 'Unlock Colors', {callback = unlock_items, callbackData = "colors"})
	PreGameMenu:addOption('inventory_menu', 'Unlock Masks', {callback = unlock_items, callbackData = "masks"})
	PreGameMenu:addOption('inventory_menu', 'Unlock Materials', {callback = unlock_items, callbackData = "materials"})
	PreGameMenu:addOption('inventory_menu', 'Unlock Textures', {callback = unlock_items, callbackData = "textures"})
	PreGameMenu:addOption('inventory_menu', 'Unlock Weapons', {callback = unlock_items, callbackData = "weapons"})
	-- Inventory Menu Column 2
	PreGameMenu:addInformationOption('inventory_menu', 'MISC. OPTIONS', {textColor = Color.DodgerBlue})
	--PreGameMenu:addToggleOption('inventory_menu', 'Remove Limit of Installed Modifications', {callback = nolimitmod})
	PreGameMenu:addOption('inventory_menu', 'Remove New Item Exclamation Mark', {callback = remexclmark})
	PreGameMenu:addGap('inventory_menu')
	PreGameMenu:addGap('inventory_menu')
	PreGameMenu:addInformationOption('inventory_menu', 'UNLOCK OPTIONS (CONT)', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('inventory_menu', 'Unlock Weapon Mods', {callback = unlock_items, callbackData = "weapon_mods"})
	
	-- Clear Inventory Menu Column 1
	PreGameMenu:addInformationOption('clear_inventory_menu', 'DELETE INDIVIDUAL SLOTS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Masks Slots', {callback = clear_slots, callbackData = "masks"})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Primary Weapons Slots', {callback = clear_slots, callbackData = "primaries"})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Secondary Weapons Slots', {callback = clear_slots, callbackData = "secondaries"})
	PreGameMenu:addGap('clear_inventory_menu')
	PreGameMenu:addInformationOption('clear_inventory_menu', 'DELETE OR LOCK ALL SLOTS AND ITEMS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Items', {callback = clearitems, callbackData = {["type"] = "all", ["globalval"] = "all"}})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Slots', {callback = clear_slots, callbackData = "all"})
	PreGameMenu:addOption('clear_inventory_menu', 'Lock All Items', {callback = delete_items})
	PreGameMenu:addGap('clear_inventory_menu')
	-- Clear Inventory Menu Column 2
	PreGameMenu:addInformationOption('clear_inventory_menu', 'DELETE INDIVIDUAL ITEMS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Colors', {callback = clearitems, callbackData = {["type"] = "colors", ["globalval"] = "all"}})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Masks', {callback = clearitems, callbackData = {["type"] = "masks", ["globalval"] = "all"}})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Materials', {callback = clearitems, callbackData = {["type"] = "materials", ["globalval"] = "all"}})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Patterns', {callback = clearitems, callbackData = {["type"] = "textures", ["globalval"] = "all"}})
	PreGameMenu:addOption('clear_inventory_menu', 'Delete All Weapon Mods', {callback = clearitems, callbackData = {["type"] = "weapon_mods", ["globalval"] = "all"}})
	for i=7,10 do
		PreGameMenu:addGap('clear_inventory_menu')
	end	
	
	-- Add To Inventory Menu Column 1
	PreGameMenu:addInformationOption('add_inventory_menu', 'ADD 1 OF EACH ITEM', {textColor = Color.red})
	PreGameMenu:addOption('add_inventory_menu', 'Add 1 Of All Colors', {callback = giveitems, callbackData = {["type"] = "colors", ["times"] = nil}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 1 Of Everything', {callback = giveitems, callbackData = {["type"] = "all", ["times"] = nil}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 1 Of All Masks', {callback = giveitems, callbackData = {["type"] = "masks", ["times"] = nil}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 1 Of All Materials', {callback = giveitems, callbackData = {["type"] = "materials", ["times"] = nil}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 1 Of All Patterns', {callback = giveitems, callbackData = {["type"] = "textures", ["times"] = nil}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 1 Of All Weapon Mods', {callback = giveitems, callbackData = {["type"] = "weapon_mods", ["times"] = nil}})
	PreGameMenu:addGap('add_inventory_menu')
	PreGameMenu:addGap('add_inventory_menu')
	PreGameMenu:addGap('add_inventory_menu')
	-- Add To Inventory Menu Column 2
	PreGameMenu:addInformationOption('add_inventory_menu', 'ADD 5 OF EACH ITEM', {textColor = Color.red})
	PreGameMenu:addOption('add_inventory_menu', 'Add 5 Of All Colors', {callback = giveitems, callbackData = {["type"] = "colors", ["times"] = 5}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 5 Of Everything', {callback = giveitems, callbackData = {["type"] = "all", ["times"] = 5}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 5 Of All Masks', {callback = giveitems, callbackData = {["type"] = "masks", ["times"] = 5}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 5 Of All Materials', {callback = giveitems, callbackData = {["type"] = "materials", ["times"] = 5}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 5 Of All Patterns', {callback = giveitems, callbackData = {["type"] = "textures", ["times"] = 5}})
	PreGameMenu:addOption('add_inventory_menu', 'Add 5 Of All Weapon Mods', {callback = giveitems, callbackData = {["type"] = "weapon_mods", ["times"] = 5}})
	PreGameMenu:addMenuOption('main_menu', 'Level Menu', 'level_menu', {rectHighlightColor = Color.red})
	
	-- Level Menu Column 1
	PreGameMenu:addInformationOption('level_menu', 'XP OPTIONS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('level_menu', 'Add 100 XP', {callback = function() add_exp(100) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Add 1,000 XP', {callback = function() add_exp(1000) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Add 10,000 XP', {callback = function() add_exp(10000) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Add 50,000 XP', {callback = function() add_exp(50000) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Add 100,000 XP', {callback = function() add_exp(100000) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Add 200,000 XP', {callback = function() add_exp(200000) end, closeMenu = true})
	PreGameMenu:addGap('level_menu')
	PreGameMenu:addGap('level_menu')
	PreGameMenu:addGap('level_menu')
	PreGameMenu:addGap('level_menu')
	-- Level Menu Column 2
	PreGameMenu:addInformationOption('level_menu', 'LEVEL OPTIONS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('level_menu', 'Set Level To 10', {callback = function() change_level(10) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Set Level To 20', {callback = function() change_level(20) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Set Level To 30', {callback = function() change_level(30) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Set Level To 40', {callback = function() change_level(40) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Set Level To 50', {callback = function() change_level(50) end, closeMenu = true})
	PreGameMenu:addGap('level_menu')
	PreGameMenu:addGap('level_menu')
	PreGameMenu:addGap('level_menu')
	PreGameMenu:addGap('level_menu')	
	-- Level Menu Column 3
	PreGameMenu:addGap('level_menu')
	PreGameMenu:addGap('level_menu')
	PreGameMenu:addOption('level_menu', 'Set Level To 60', {callback = function() change_level(60) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Set Level To 70', {callback = function() change_level(70) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Set Level To 80', {callback = function() change_level(80) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Set Level To 90', {callback = function() change_level(90) end, closeMenu = true})
	PreGameMenu:addOption('level_menu', 'Set Level To 100', {callback = function() change_level(100) end, closeMenu = true})
	
	PreGameMenu:addMenuOption('main_menu', 'Money Menu', 'money_menu', {rectHighlightColor = Color.red})
	-- Money Menu Column 1
	PreGameMenu:addInformationOption('money_menu', 'RESET MONEY OPTION', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('money_menu','Reset Money', {callback = reset_money})
	PreGameMenu:addGap('money_menu')
	PreGameMenu:addInformationOption('money_menu', 'ADD MONEY OPTION', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('money_menu', 'Add 1 Million Cash + 4 Million Offshore', {callback = function() add_money(5000000) end, closeMenu = true})
	PreGameMenu:addOption('money_menu', 'Add 10 Million Cash + 40 Million Offshore', {callback = function() add_money(50000000) end, closeMenu = true})
	PreGameMenu:addOption('money_menu', 'Add 100 Million Cash + 400 Million Offshore', {callback = function() add_money(500000000) end, closeMenu = true})
	PreGameMenu:addOption('money_menu', 'Add 1 Billion Cash + 4 Billion Offshore', {callback = function() add_money(5000000000) end, closeMenu = true})
	
	PreGameMenu:addMenuOption('main_menu', 'Skill And Perk Menu', 'skill_menu', {rectHighlightColor = Color.red})
	-- Skill Menu Column 1
	PreGameMenu:addInformationOption('skill_menu', 'SKILLS OPTIONS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('skill_menu','Reset Skill Points', {callback = function() set_skillpoints(0) end, closeMenu = true})
	PreGameMenu:addOption('skill_menu','Unlock All Skills', {callback = function() for i=1,2 do unlock_all_skills() end end, closeMenu = true})
	PreGameMenu:addGap('skill_menu')
	PreGameMenu:addInformationOption('skill_menu', 'SKILL POINTS OPTIONS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('skill_menu', 'Set Skill Points To 120', {callback = function() set_skillpoints(120) end, help = 'Unlock One Full Tree', closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Set Skill Points To 290', {callback = function() set_skillpoints(290) end, help = 'Unlock Two Full Trees', closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Set Skill Points To 435', {callback = function() set_skillpoints(435) end, help = 'Unlock Three Full Trees', closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Set Skill Points To 580', {callback = function() set_skillpoints(580) end, help = 'Unlock Four Full Trees', closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Set Skill Points To 725', {callback = function() set_skillpoints(725) end, help = 'Unlock All Trees', closeMenu = true})
	-- Skill Menu Column 2
	PreGameMenu:addInformationOption('skill_menu', 'PERK OPTIONS', {textColor = Color.DodgerBlue})	
	PreGameMenu:addOption('skill_menu', 'Reset Perk Points', {callback = reset_perkpoints, closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Unlock All Perks', {callback = unlockperks, closeMenu = true, help = 'No Visual Effect On Perk Decks, Go In Game To See Active Perks'})
	PreGameMenu:addGap('skill_menu')
	PreGameMenu:addInformationOption('skill_menu', 'PERK POINTS OPTIONS', {textColor = Color.DodgerBlue})
	PreGameMenu:addOption('skill_menu', 'Add 500 To Perk Points', {callback = function() set_perkpoints(500000) end, closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Add 1,000 To Perk Points', {callback = function() set_perkpoints(1000000) end, closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Add 2,500 To Perk Points', {callback = function() set_perkpoints(2500000) end, closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Add 5,000 To Perk Points', {callback = function() set_perkpoints(5000000) end, closeMenu = true})
	PreGameMenu:addOption('skill_menu', 'Add 10,000 To Perk Points', {callback = function() set_perkpoints(10000000) end, closeMenu = true})
	
	-- Options
	PreGameMenu:addGap('main_menu')
	PreGameMenu:addInformationOption('main_menu', 'ACHIEVMENT OPTIONS', {textColor = Color.DodgerBlue})
	-- Add Achievements
	PreGameMenu:addMenuOption('main_menu', 'Add Individual Achievements', 'add_achieve_menu', {rectHighlightColor = Color.red})
	for id,name in pairs(achieveid) do
		PreGameMenu:addOption('add_achieve_menu', name, {callback = addachieve, callbackData = id})
	end
	-- Remove Achievements
	PreGameMenu:addMenuOption('main_menu', 'Remove Individual Achievements', 'rem_achieve_menu', {rectHighlightColor = Color.red})
	for id,name in pairs(achieveid) do
		PreGameMenu:addOption('rem_achieve_menu', name, {callback = remachieve, callbackData = id})
	end
	for i=1,9 do
		PreGameMenu:addGap('main_menu')
    end
	PreGameMenu:addOption('main_menu', 'Lock All Achievements', {callback = lock_achievements, closeMenu = true})
	PreGameMenu:addOption('main_menu', 'Unlock All Achievements', {callback = unlock_achievements, closeMenu = true})
end

if not inGame() and not isPlaying() then	
	if PreGameMenu:isOpen() then
		PreGameMenu:close()
	else
		if JobSelectMenu then
			if JobSelectMenu:isOpen() then
				JobSelectMenu:close()
			end
		end
		if DifficultyMenu then
			if DifficultyMenu:isOpen() then
				DifficultyMenu:close()
			end
		end
		PreGameMenu:open()
	end	
end
-------------------------------------------------------------------------------------------------------------------------------
if not JobMenu then	
	-- JOB MENU	
	-- ALL AI
	spooccuffer = spooccuffer or function()
		function TeamAIMovement:on_SPOOCed()
			self._unit:brain():set_logic("surrender")
			self._unit:network():send("arrested")
			self._unit:character_damage():on_arrested()
	end	end
	
	-- DALLAS / RUSSIAN
	dallasnormalspeed = dallasnormalspeed or function()
		tweak_data.character.russian.move_speed = tweak_data.character.presets.move_speed.fast
	end
	dallasfastspeed = dallasfastspeed or function()
		tweak_data.character.russian.move_speed = tweak_data.character.presets.move_speed.lightning
	end
	dallasdodge = dallasdodge or function()
		tweak_data.character.russian.dodge = tweak_data.character.presets.dodge.athletic
	end
	dallasguard = dallasguard or function()
		tweak_data.character.russian.deathguard = nil
	end
	dallasweaponset1 = dallasweaponset1 or function()
		tweak_data.character.russian.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_lmg_m249/wpn_npc_lmg_m249"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mac11/wpn_npc_mac11")
		}
	end
	dallasweaponset2 = dallasweaponset2 or function()
		tweak_data.character.russian.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_sawnoff_shotgun/wpn_npc_sawnoff_shotgun"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_raging_bull/wpn_npc_raging_bull")
		}
	end
	dallasweaponset3 = dallasweaponset3 or function()
		tweak_data.character.russian.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_g36/wpn_npc_g36"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mp5_tactical/wpn_npc_mp5_tactical")
		}
	end

	-- WOLF / GERMAN
	wolfnormalspeed = wolfnormalspeed or function()
		tweak_data.character.german.move_speed = tweak_data.character.presets.move_speed.fast
	end
	wolffastspeed = wolffastspeed or function()
		tweak_data.character.german.move_speed = tweak_data.character.presets.move_speed.lightning
	end
	wolfdodge = wolfdodge or function()
		tweak_data.character.german.dodge = tweak_data.character.presets.dodge.athletic
	end
	wolfguard = wolfguard or function()
		tweak_data.character.german.deathguard = nil
	end
	wolfweaponset1 = wolfweaponset1 or function()
		tweak_data.character.german.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_lmg_m249/wpn_npc_lmg_m249"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mac11/wpn_npc_mac11")
		}
	end
	wolfweaponset2 = wolfweaponset2 or function()
		tweak_data.character.german.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_sawnoff_shotgun/wpn_npc_sawnoff_shotgun"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_raging_bull/wpn_npc_raging_bull")
		}
	end
	wolfweaponset3 = wolfweaponset3 or function()
		tweak_data.character.german.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_g36/wpn_npc_g36"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mp5_tactical/wpn_npc_mp5_tactical")
		}
	end

	-- HOXTON / AMERICAN
	hoxtonnormalspeed = hoxtonnormalspeed or function()
		tweak_data.character.american.move_speed = tweak_data.character.presets.move_speed.fast
	end
	hoxtonfastspeed = hoxtonfastspeed or function()
		tweak_data.character.american.move_speed = tweak_data.character.presets.move_speed.lightning
	end
	hoxtondodge = hoxtondodge or function()
		tweak_data.character.american.dodge = tweak_data.character.presets.dodge.athletic
	end
	hoxtonguard = hoxtonguard or function()
		tweak_data.character.american.deathguard = nil
	end
	hoxtonweaponset1 = hoxtonweaponset1 or function()
		tweak_data.character.american.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_lmg_m249/wpn_npc_lmg_m249"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mac11/wpn_npc_mac11")
		}
	end
	hoxtonweaponset2 = hoxtonweaponset2 or function()
		tweak_data.character.american.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_sawnoff_shotgun/wpn_npc_sawnoff_shotgun"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_raging_bull/wpn_npc_raging_bull")
		}
	end
	hoxtonweaponset3 = hoxtonweaponset3 or function()
		tweak_data.character.american.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_g36/wpn_npc_g36"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mp5_tactical/wpn_npc_mp5_tactical")
		}
	end

	-- CHAINS / SPANISH
	chainsnormalspeed = chainsnormalspeed or function()
		tweak_data.character.spanish.move_speed = tweak_data.character.presets.move_speed.fast
	end
	chainsfastspeed = chainsfastspeed or function()
		tweak_data.character.spanish.move_speed = tweak_data.character.presets.move_speed.lightning
	end
	chainsdodge = chainsdodge or function()
		tweak_data.character.spanish.dodge = tweak_data.character.presets.dodge.athletic
	end
	chainsguard = chainsguard or function()
		tweak_data.character.spanish.deathguard = nil
	end
	chainsweaponset1 = chainsweaponset1 or function()
		tweak_data.character.spanish.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_lmg_m249/wpn_npc_lmg_m249"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mac11/wpn_npc_mac11")
		}
	end
	chainsweaponset2 = chainsweaponset2 or function()
		tweak_data.character.spanish.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_sawnoff_shotgun/wpn_npc_sawnoff_shotgun"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_raging_bull/wpn_npc_raging_bull")
		}
	end
	chainsweaponset3 = chainsweaponset3 or function()
		tweak_data.character.spanish.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_g36/wpn_npc_g36"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mp5_tactical/wpn_npc_mp5_tactical")
		}
	end

    -- JOHN WICK / JOWI
	wicknormalspeed = wicknormalspeed or function()
		tweak_data.character.jowi.move_speed = tweak_data.character.presets.move_speed.fast
	end
	wickfastspeed = wickfastspeed or function()
		tweak_data.character.jowi.move_speed = tweak_data.character.presets.move_speed.lightning
	end
	wickdodge = wickdodge or function()
		tweak_data.character.jowi.dodge = tweak_data.character.presets.dodge.athletic
	end
	wickguard = wickguard or function()
		tweak_data.character.jowi.deathguard = nil
	end
	wickweaponset1 = wickweaponset1 or function()
		tweak_data.character.jowi.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_lmg_m249/wpn_npc_lmg_m249"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mac11/wpn_npc_mac11")
		}
	end
	wickweaponset2 = wickweaponset2 or function()
		tweak_data.character.jowi.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_sawnoff_shotgun/wpn_npc_sawnoff_shotgun"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_raging_bull/wpn_npc_raging_bull")
		}
	end
	wickweaponset3 = wickweaponset3 or function()
		tweak_data.character.jowi.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_g36/wpn_npc_g36"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mp5_tactical/wpn_npc_mp5_tactical")
		}
	end
	
	-- OLD HOXTON / OLD_HOXTON
	oldhoxnormalspeed = oldhoxnormalspeed or function()
		tweak_data.character.old_hoxton.move_speed = tweak_data.character.presets.move_speed.fast
	end
	oldhoxfastspeed = oldhoxfastspeed or function()
		tweak_data.character.old_hoxton.move_speed = tweak_data.character.presets.move_speed.lightning
	end
	oldhoxdodge = oldhoxdodge or function()
		tweak_data.character.old_hoxton.dodge = tweak_data.character.presets.dodge.athletic
	end
	oldhoxguard = oldhoxguard or function()
		tweak_data.character.old_hoxton.deathguard = nil
	end
	oldhoxweaponset1 = oldhoxweaponset1 or function()
		tweak_data.character.old_hoxton.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_lmg_m249/wpn_npc_lmg_m249"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mac11/wpn_npc_mac11")
		}
	end
	oldhoxweaponset2 = oldhoxweaponset2 or function()
		tweak_data.character.old_hoxton.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_sawnoff_shotgun/wpn_npc_sawnoff_shotgun"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_raging_bull/wpn_npc_raging_bull")
		}
	end
	oldhoxweaponset3 = oldhoxweaponset3 or function()
		tweak_data.character.old_hoxton.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_g36/wpn_npc_g36"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mp5_tactical/wpn_npc_mp5_tactical")
		}
	end
	
	-- CLOVER/ FEMALE_1
	clovernormalspeed = clovernormalspeed or function()
		tweak_data.character.female_1.move_speed = tweak_data.character.presets.move_speed.fast
	end
	cloverfastspeed = cloverfastspeed or function()
		tweak_data.character.female_1.move_speed = tweak_data.character.presets.move_speed.lightning
	end
	cloverdodge = cloverdodge or function()
		tweak_data.character.female_1.dodge = tweak_data.character.presets.dodge.athletic
	end
	cloverguard = cloverguard or function()
		tweak_data.character.female_1.deathguard = nil
	end
	cloverweaponset1 = cloverweaponset1 or function()
		tweak_data.character.female_1.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_lmg_m249/wpn_npc_lmg_m249"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mac11/wpn_npc_mac11")
		}
	end
	cloverweaponset2 = cloverweaponset2 or function()
		tweak_data.character.female_1.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_sawnoff_shotgun/wpn_npc_sawnoff_shotgun"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_raging_bull/wpn_npc_raging_bull")
		}
	end
	cloverweaponset3 = cloverweaponset3 or function()
		tweak_data.character.female_1.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_g36/wpn_npc_g36"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mp5_tactical/wpn_npc_mp5_tactical")
		}
	end	
	
	-- DRAGAN/ DRAGAN
	dragannormalspeed = dragannormalspeed or function()
		tweak_data.character.dragan.move_speed = tweak_data.character.presets.move_speed.fast
	end
	draganfastspeed = draganfastspeed or function()
		tweak_data.character.dragan.move_speed = tweak_data.character.presets.move_speed.lightning
	end
	dragandodge = dragandodge or function()
		tweak_data.character.dragan.dodge = tweak_data.character.presets.dodge.athletic
	end
	draganguard = draganguard or function()
		tweak_data.character.dragan.deathguard = nil
	end
	draganweaponset1 = draganweaponset1 or function()
		tweak_data.character.dragan.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_lmg_m249/wpn_npc_lmg_m249"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mac11/wpn_npc_mac11")
		}
	end
	draganweaponset2 = draganweaponset2 or function()
		tweak_data.character.dragan.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_sawnoff_shotgun/wpn_npc_sawnoff_shotgun"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_raging_bull/wpn_npc_raging_bull")
		}
	end
	draganweaponset3 = draganweaponset3 or function()
		tweak_data.character.dragan.weapon.weapons_of_choice = {
			primary = Idstring("units/payday2/weapons/wpn_npc_g36/wpn_npc_g36"), 
			secondary = Idstring("units/payday2/weapons/wpn_npc_mp5_tactical/wpn_npc_mp5_tactical")
		}
	end

	buffemall = function()
		-- SPOOCCUFFS
		spooccuffer()
		-- AI MOVESPEED
		dallasfastspeed()
		wolffastspeed()
		hoxtonfastspeed()
		chainsfastspeed()
		wickfastspeed()
		oldhoxfastspeed()
		cloverfastspeed()
		draganfastspeed()
		-- AI DODGE
		dallasdodge()
		wolfdodge()
		hoxtondodge()
		chainsdodge()
		wickdodge()
		oldhoxdodge()
		cloverdodge()
		dragandodge()
		-- AI WEAPONSET
		dallasweaponset1()
		wolfweaponset1()
		hoxtonweaponset1()
		chainsweaponset1()
		wickweaponset1()
		oldhoxweaponset1()
		cloverweaponset1()
		draganweaponset1()
		-- AI DEATHGUARD
		dallasguard()
		wolfguard()
		hoxtonguard()
		chainsguard()
		wickguard()
		oldhoxguard()
		cloverguard()
		draganguard()
	end
	
	-- DISABLE ESCAPE TIMER by Harfatus
	notimer = notimer or function()
		function ElementPointOfNoReturn:on_executed(...) end
	end
	
	-- PREVENT WAITING FOR PLAYERS AT START
	dontwait = dontwait or function()
		if managers.network:game() then
			managers.network:game():spawn_players()
		end	
	end
	
	-- UNLOCK ALL ASSETS
	getemall = getemall or function()
		if inGame() then
			for _,asset_id in pairs(managers.assets:get_all_asset_ids(true)) do
				managers.assets:unlock_asset(asset_id)
			end	
		end	
	end
-------------------------------------------------------------------------------
	JobMenu = CustomMenuClass:new()
	JobMenu:addMainMenu('main_menu', {title = 'Job Menu - Select Menu Or Option'})
	JobMenu:addMenu('ai_menu', {title = 'Team AI Menu - Select Option'})
	JobMenu:addMenu('chains_menu', {title = 'Chains Menu - Select Option'})
	JobMenu:addMenu('dallas_menu', {title = 'Dallas Menu - Select Option'})
	JobMenu:addMenu('hoxton_menu', {title = 'Hoxton Menu - Select Option'})
	JobMenu:addMenu('wolf_menu', {title = 'Wolf Menu - Select Option'})
	JobMenu:addMenu('wick_menu', {title = 'John Wick Menu - Select Option'})
	JobMenu:addMenu('oldhox_menu', {title = 'Old Hoxton Menu - Select Option'})
	JobMenu:addMenu('clover_menu', {title = 'Clover Menu - Select Option'})
	JobMenu:addMenu('dragan_menu', {title = 'Dragan Menu - Select Option'})
	
	-- Job Menu
	JobMenu:addInformationOption('main_menu', 'TEAM AI MENU', {textColor = Color.red})
	if isHost() then
		JobMenu:addMenuOption('main_menu', 'Team AI Options Menu', 'ai_menu', {rectHighlightColor = Color.red})
	else
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
	end
	JobMenu:addGap('main_menu')
	JobMenu:addInformationOption('main_menu', 'PREGAME OPTIONS', {textColor = Color.DodgerBlue})
	if isHost() then	
		JobMenu:addOption('main_menu', 'Force Start',  {callback = dontwait, closeMenu = true, help = 'Force Start The Game, Prevent Waiting'})
	else
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
	end
	JobMenu:addOption('main_menu', 'Unlock All Assets',  {callback =  getemall, closeMenu = true})
	JobMenu:addGap('main_menu')
	JobMenu:addGap('main_menu')
	JobMenu:addGap('main_menu')
	JobMenu:addGap('main_menu')
	
	-- Job Menu Column 2
	JobMenu:addGap('main_menu')
	JobMenu:addGap('main_menu')
	JobMenu:addGap('main_menu')
	JobMenu:addInformationOption('main_menu', 'IN GAME OPTIONS', {textColor = Color.DodgerBlue})
	if MVPConfig.Auto_Cheats then
		JobMenu:addInformationOption('main_menu','Auto Cheats/Scripts (Enabled)', {textColor = Color.yellow})
	else
		JobMenu:addToggleOption('main_menu','Auto Cheats/Scripts', {callback = function() Toggle.auto_cheats = not Toggle.auto_cheats end, help = 'Custom Selection Of Activated Scipts'})
	end
	if isHost() then
		if MVPConfig.Auto_Loot then
			JobMenu:addInformationOption('main_menu','Auto Loot (Enabled)', {textColor = Color.yellow})
		else
			JobMenu:addToggleOption('main_menu','Auto Loot', {callback = function() Toggle.auto_loot = not Toggle.auto_loot end, help = 'Spawns Randomized Loot'})
		end
		if MVPConfig.Auto_Equipment then
			JobMenu:addInformationOption('main_menu','Auto Special Equipment (Enabled)', {textColor = Color.yellow})
		else
			JobMenu:addToggleOption('main_menu','Auto Special Equipment', {callback = function() Toggle.auto_equip = not Toggle.auto_equip end, help = 'Spawns Randomized Keycards'})
		end
		if MVPConfig.Secure_BodyBags then
			JobMenu:addInformationOption('main_menu','Secure Body Bags For Cash (Enabled)', {textColor = Color.yellow})
		else
			JobMenu:addToggleOption('main_menu','Secure Body Bags For Cash', {callback = function() Toggle.body_bag_secure = not Toggle.body_bag_secure end, help = 'Allows You To Secure Body Bags'})
		end
	else
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
	end
	if MVPConfig.Toggle_Interact then
		JobMenu:addInformationOption('main_menu','Toggle Interact (Enabled)', {textColor = Color.yellow})
	else
		JobMenu:addToggleOption('main_menu','Toggle Interact', {callback = function() Toggle.toggle_interact = not Toggle.toggle_interact end, help = 'Interaction Toggle', toggle = false})
	end
	JobMenu:addGap('main_menu')
	
	-- Job Menu Column 3
	local level = managers.job:current_level_id()
	JobMenu:addGap('main_menu')
	JobMenu:addGap('main_menu')
	JobMenu:addGap('main_menu')
	JobMenu:addGap('main_menu')
	if isHost() then
		if MVPConfig.Grenade_Restrict then
			JobMenu:addInformationOption('main_menu', 'Grenade Restriction (Enabled)',  {textColor = Color.yellow})
		else	
			JobMenu:addToggleOption('main_menu', 'Grenade Restriction',  {callback = restrictgrenade, help = 'Restricts Grenades Being Thrown While In Stealth'})
		end
		JobMenu:addOption('main_menu', 'No Escape Timer',  {callback = notimer, closeMenu = true})
		if MVPConfig.Loot_Value_Mod then
			JobMenu:addInformationOption('main_menu', 'Loot Value Modifier (Enabled)',  {textColor = Color.yellow})
		else
			JobMenu:addToggleOption('main_menu', 'Loot Value Modifier',  {callback = function() Toggle.loot_modifier = not Toggle.loot_modifier end, help = 'User Defined Loot Values'})
		end
		if MVPConfig.Pager_Mod then
			if level == 'gallery' or level == 'kosugi' or level == 'firestarter_2' or level == 'big' or level == 'welcome_to_the_jungle_2' or level == 'election_day_1' or level == 'election_day_2' or level == 'election_day_3_skip1' or level == 'framing_frame_1' or level == 'framing_frame_3' or level == 'mallcrasher' then
				JobMenu:addInformationOption('main_menu', 'Pager Mod (Enabled)',  {textColor = Color.yellow})
			end
		else
			if level == 'gallery' or level == 'kosugi' or level == 'firestarter_2' or level == 'big' or level == 'welcome_to_the_jungle_2' or level == 'election_day_1' or level == 'election_day_2' or level == 'election_day_3_skip1' or level == 'framing_frame_1' or level == 'framing_frame_3' or level == 'mallcrasher' then
				JobMenu:addToggleOption('main_menu', 'Pager Mod',  {callback = function() Toggle.pager_mod = not Toggle.pager_mod end, help = 'Increases Pager Amount'})
			end
		end
		if MVPConfig.Upgrade_Tweaks then
			JobMenu:addInformationOption('main_menu', 'Upgrade Tweaks (Enabled)',  {textColor = Color.yellow})
		else	
			JobMenu:addToggleOption('main_menu', 'Upgrade Tweaks',  {callback = function() Toggle.upgrade_tweak = not Toggle.upgrade_tweak end, help = 'User Defined Upgrade Tweaks'})
		end
	else
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
		if not level == 'gallery' or level == 'kosugi' or level == 'firestarter_2' or level == 'big' or level == 'welcome_to_the_jungle_2' or level == 'election_day_1' or level == 'election_day_2' or level == 'election_day_3_skip1' or level == 'framing_frame_1' or level == 'framing_frame_3' or level == 'mallcrasher' then
			JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
		end
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
		JobMenu:addInformationOption('main_menu', 'Not Available (Host Only)', {textColor = Color.yellow})
	end
	if MVPConfig.XP_Multiplier then
		JobMenu:addInformationOption('main_menu', 'XP Multiplier (Enabled)', {textColor = Color.yellow})
	else	
		JobMenu:addToggleOption('main_menu', 'XP Multiplier',  {callback = xpmultiplier, help = 'Multiply XP Earned By 2x, Enable For Each Map You Want To Multiply'})
	end
	
	-- AI Menu
	JobMenu:addInformationOption('ai_menu', 'AI MENUS', {textColor = Color.red})
	-- Chains Menu
	JobMenu:addMenuOption('ai_menu', 'Chains Menu', 'chains_menu', {rectHighlightColor = Color.red})
	JobMenu:addInformationOption('chains_menu', 'CHAINS OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('chains_menu', 'Enable Dodge Bullets', {callback = chainsdodge})
	JobMenu:addOption('chains_menu', 'Heal Player Is Primary Goal', {callback = chainsguard})
	JobMenu:addOption('chains_menu', 'Speed Normal', {callback = chainsnormalspeed})
	JobMenu:addOption('chains_menu', 'Speed Fast', {callback = chainsfastspeed})
	JobMenu:addGap('chains_menu')
	JobMenu:addInformationOption('chains_menu', 'CHAINS WEAPON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('chains_menu', 'M249 / MAC11', {callback = chainsweaponset1})
	JobMenu:addOption('chains_menu', 'Sawnoff Shotgun / .45', {callback = chainsweaponset2})
	JobMenu:addOption('chains_menu', 'G36 / MP5', {callback = chainsweaponset3})
	
	-- Dallas Menu
	JobMenu:addMenuOption('ai_menu', 'Dallas Menu', 'dallas_menu', {rectHighlightColor = Color.red})
	JobMenu:addInformationOption('dallas_menu', 'DALLAS OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('dallas_menu', 'Enable Dodge Bullets', {callback = dallasdodge})
	JobMenu:addOption('dallas_menu', 'Heal Player Is Primary Goal', {callback = dallasguard})
	JobMenu:addOption('dallas_menu', 'Speed Normal', {callback = dallasnormalspeed})
	JobMenu:addOption('dallas_menu', 'Speed Fast', {callback = dallasfastspeed})
	JobMenu:addGap('dallas_menu')
	JobMenu:addInformationOption('dallas_menu', 'DALLAS WEAPON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('dallas_menu', 'M249 / MAC11', {callback = dallasweaponset1})
	JobMenu:addOption('dallas_menu', 'Sawnoff Shotgun / .45', {callback = dallasweaponset2})
	JobMenu:addOption('dallas_menu', 'G36 / MP5', {callback = dallasweaponset3})
	
	-- Hoxton Menu
	JobMenu:addMenuOption('ai_menu', 'Hoxton Menu', 'hoxton_menu', {rectHighlightColor = Color.red})
	JobMenu:addInformationOption('hoxton_menu', 'HOXTON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('hoxton_menu', 'Enable Dodge Bullets', {callback = hoxtondodge})
	JobMenu:addOption('hoxton_menu', 'Heal Player Is Primary Goal', {callback = hoxtonguard})
	JobMenu:addOption('hoxton_menu', 'Speed Normal', {callback = hoxtonnormalspeed})
	JobMenu:addOption('hoxton_menu', 'Speed Fast', {callback = hoxtonfastspeed})
	JobMenu:addGap('hoxton_menu')
	JobMenu:addInformationOption('hoxton_menu', 'HOXTON WEAPON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('hoxton_menu', 'M249 / MAC11', {callback = hoxtonweaponset1})
	JobMenu:addOption('hoxton_menu', 'Sawnoff Shotgun / .45', {callback = hoxtonweaponset2})
	JobMenu:addOption('hoxton_menu', 'G36 / MP5', {callback = hoxtonweaponset3})
	
	-- Wolf Menu
	JobMenu:addMenuOption('ai_menu', 'Wolf Menu', 'wolf_menu', {rectHighlightColor = Color.red})
	JobMenu:addInformationOption('wolf_menu', 'WOLF OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('wolf_menu', 'Enable Dodge Bullets', {callback = wolfdodge})
	JobMenu:addOption('wolf_menu', 'Heal Player Is Primary Goal', {callback = wolfguard})
	JobMenu:addOption('wolf_menu', 'Speed Normal', {callback = wolfnormalspeed})
	JobMenu:addOption('wolf_menu', 'Speed Fast', {callback = wolffastspeed})
	JobMenu:addGap('wolf_menu')
	JobMenu:addInformationOption('wolf_menu', 'WOLF WEAPON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('wolf_menu', 'M249 / MAC11', {callback = wolfweaponset1})
	JobMenu:addOption('wolf_menu', 'Sawnoff Shotgun / .45', {callback = wolfweaponset2})
	JobMenu:addOption('wolf_menu', 'G36 / MP5', {callback = wolfweaponset3})
	
	-- John Wick Menu
	JobMenu:addMenuOption('ai_menu', 'John Wick Menu', 'wick_menu', {rectHighlightColor = Color.red})
	JobMenu:addInformationOption('wick_menu', 'JOHN WICK OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('wick_menu', 'Enable Dodge Bullets', {callback = wickdodge})
	JobMenu:addOption('wick_menu', 'Heal Player Is Primary Goal', {callback = wickguard})
	JobMenu:addOption('wick_menu', 'Speed Normal', {callback = wicknormalspeed})
	JobMenu:addOption('wick_menu', 'Speed Fast', {callback = wickfastspeed})
	JobMenu:addGap('wick_menu')
	JobMenu:addInformationOption('wick_menu', 'JOHN WICK WEAPON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('wick_menu', 'M249 / MAC11', {callback = wickweaponset1})
	JobMenu:addOption('wick_menu', 'Sawnoff Shotgun / .45', {callback = wickweaponset2})
	JobMenu:addOption('wick_menu', 'G36 / MP5', {callback = wickweaponset3})
	
	-- Old Hoxton Menu
	JobMenu:addMenuOption('ai_menu', 'Old Hoxton Menu', 'oldhox_menu', {rectHighlightColor = Color.red})
	JobMenu:addInformationOption('oldhox_menu', 'OLD HOXTON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('oldhox_menu', 'Enable Dodge Bullets', {callback = oldhoxdodge})
	JobMenu:addOption('oldhox_menu', 'Heal Player Is Primary Goal', {callback = oldhoxguard})
	JobMenu:addOption('oldhox_menu', 'Speed Normal', {callback = oldhoxnormalspeed})
	JobMenu:addOption('oldhox_menu', 'Speed Fast', {callback = oldhoxfastspeed})
	JobMenu:addGap('oldhox_menu')
	JobMenu:addInformationOption('oldhox_menu', 'OLD HOXTON WEAPON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('oldhox_menu', 'M249 / MAC11', {callback = oldhoxweaponset1})
	JobMenu:addOption('oldhox_menu', 'Sawnoff Shotgun / .45', {callback = oldhoxweaponset2})
	JobMenu:addOption('oldhox_menu', 'G36 / MP5', {callback = oldhoxweaponset3})
	
	-- Clover Menu
	JobMenu:addMenuOption('ai_menu', 'Clover Menu', 'clover_menu', {rectHighlightColor = Color.red})
	JobMenu:addInformationOption('clover_menu', 'CLOVER OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('clover_menu', 'Enable Dodge Bullets', {callback = cloverdodge})
	JobMenu:addOption('clover_menu', 'Heal Player Is Primary Goal', {callback = cloverguard})
	JobMenu:addOption('clover_menu', 'Speed Normal', {callback = clovernormalspeed})
	JobMenu:addOption('clover_menu', 'Speed Fast', {callback = cloverfastspeed})
	JobMenu:addGap('clover_menu')
	JobMenu:addInformationOption('clover_menu', 'CLOVER WEAPON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('clover_menu', 'M249 / MAC11', {callback = cloverweaponset1})
	JobMenu:addOption('clover_menu', 'Sawnoff Shotgun / .45', {callback = cloverweaponset2})
	JobMenu:addOption('clover_menu', 'G36 / MP5', {callback = cloverweaponset3})	
	
	-- Dragan Menu
	JobMenu:addMenuOption('ai_menu', 'Dragan Menu', 'dragan_menu', {rectHighlightColor = Color.red})
	JobMenu:addInformationOption('dragan_menu', 'DRAGAN OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('dragan_menu', 'Enable Dodge Bullets', {callback = dragandodge})
	JobMenu:addOption('dragan_menu', 'Heal Player Is Primary Goal', {callback = draganguard})
	JobMenu:addOption('dragan_menu', 'Speed Normal', {callback = dragannormalspeed})
	JobMenu:addOption('dragan_menu', 'Speed Fast', {callback = draganfastspeed})
	JobMenu:addGap('dragan_menu')
	JobMenu:addInformationOption('dragan_menu', 'DRAGAN WEAPON OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('dragan_menu', 'M249 / MAC11', {callback = draganweaponset1})
	JobMenu:addOption('dragan_menu', 'Sawnoff Shotgun / .45', {callback = draganweaponset2})
	JobMenu:addOption('dragan_menu', 'G36 / MP5', {callback = draganweaponset3})
	
	-- AI MENU
	JobMenu:addGap('ai_menu')
	JobMenu:addInformationOption('ai_menu', 'AI OPTIONS', {textColor = Color.DodgerBlue})
	JobMenu:addOption('ai_menu', 'All AI Buffs On', {callback = buffemall})
	JobMenu:addOption('ai_menu', 'Cloaker Only Cuffs AI', {callback = spooccuffer})
end

if inGame() and not isPlaying() then	
	if JobMenu:isOpen() then
		JobMenu:close()
	else
		JobMenu:open()
	end	
end
------------------------------------------------------------------------------------------------------------------------
-- INGAME
-- HIDE HUD TOGGLE
if MVPConfig.Hud_Toggle then
	if isPlaying() then
		Toggle.hudvisible = not Toggle.hudvisible

		if not Toggle.hudvisible then
			managers.hud:set_enabled()
			showHint('HUD ENABLED', 2)
			return
		end
		
		managers.hud:set_disabled()
	end
end