require 'MVP/UserConfig/Config'

local SHOW_TIMERS = MVPConfig.Lazy_Show_Timer	--Drills, timelocks, hacking etc.
local SHOW_EQUIPMENT = MVPConfig.Lazy_Show_Equipment --Deployables (ammo, doc bags, body bags)
local SHOW_MINIONS = MVPConfig.Lazy_Show_Minions --Converted enemies, type and health
local SHOW_PAGERS = MVPConfig.Lazy_Show_Pagers	--Active pagers
local SHOW_ECMS = MVPConfig.Lazy_Show_ECMS	--Active ECMs
local SHOW_ENEMIES = MVPConfig.Lazy_Show_Enemies	--Currently spawned enemies
local AGGREGATE_ENEMIES = MVPConfig.Lazy_Aggregate_Enemies	--Don't split enemies on type; use a single entry for all
local SHOW_CIVILIANS = MVPConfig.Lazy_Show_Civilians	--Currently spawned, untied civs
local SHOW_HOSTAGES = MVPConfig.Lazy_Show_Hostages	--Currently tied civilian and dominated cops
local SHOW_BUFFS = MVPConfig.Lazy_Show_Buffs	--Active effects (buffs/debuffs). Also see the DynamicHUDList.BuffItemBase.BUFFS list, where you can set the ignore flag to not show specific things)
local SHOW_LOOT = MVPConfig.Lazy_Show_Loot --Show spawned and active loot bags/piles (may not be shown if certain mission parameters has not been met)
local AGGREGATE_LOOT = MVPConfig.Lazy_Aggregate_Loot --Don't split loot on type; use a single entry for all
local LOOT_DEBUG = false

if MVPConfig.Lazy_Hud then
	if string.lower(RequiredScript) == "lib/managers/hudmanagerpd2" then

		local HUDManager_update_original = HUDManager.update
		local HUDManager_setup_player_info_hud_pd2_original = HUDManager._setup_player_info_hud_pd2

		function HUDManager:_setup_player_info_hud_pd2(...)
			HUDManager_setup_player_info_hud_pd2_original(self, ...)
			
			if SHOW_TIMERS or SHOW_EQUIPMENT or SHOW_MINIONS or SHOW_PAGERS then
				self:_setup_left_list()
			end
			
			if SHOW_ENEMIES or SHOW_CIVILIANS or SHOW_HOSTAGES or SHOW_ECMS then
				self:_setup_right_list()
			end
			
			if SHOW_BUFFS then
				self:_setup_buff_list()
			end
			
			self._lists_initialized = true
		end
		
		function HUDManager:update(t, dt, ...)
			for _, list in pairs(self._hud_panel_lists or {}) do
				list:update(t, dt)
				if list:is_animating() then
					list:animate(t, dt)
				end
			end
			
			return HUDManager_update_original(self, t, dt, ...)
		end

		function HUDManager:list_initialized()
			return self._lists_initialized
		end
		
		function HUDManager:register_hud_list(name, class, params, ...)
			self._hud_panel_lists = self._hud_panel_lists or {}
			
			if not self._hud_panel_lists[name] then
				self._hud_panel_lists[name] = _G.DynamicHUDList[class] and _G.DynamicHUDList[class]:new(nil, name, params, ...)
			end
			
			return self._hud_panel_lists[name]
		end

		function HUDManager:get_hud_list(name)
			return self._hud_panel_lists and self._hud_panel_lists[name] or nil
		end
		
		function HUDManager:_setup_left_list()
			local list_width = 600
			local list_height = 800
			local x = 0
			local y = 40
			local list = managers.hud:register_hud_list("side_list", "VerticalList", { align = "left", x = x, y = y, w = list_width, h = list_height, top_to_bottom = true, item_margin = 5 })
		
			if SHOW_TIMERS then
				list:register_item("timers", "HorizontalList", { align = "top", w = list_width, h = 40, left_to_right = true, item_margin = 5 })
				list:item("timers"):set_static_item("LeftListIcon", 1, 4/5, { 
					{ atlas = true, texture_rect = { 3 * 64, 6 * 64, 64, 64 } },
				})
			end
			if SHOW_EQUIPMENT then
				local icons = DynamicHUDList.EquipmentItem.ICON_TABLE
				list:register_item("equipment", "HorizontalList", { align = "top", w = list_width, h = 40, left_to_right = true, item_margin = 5 })
				list:item("equipment"):set_static_item("LeftListIcon", 1, 1, { 
					{ atlas = true, h = 2/3, w = 2/3, texture_rect = { icons.ammo_bag.x, icons.ammo_bag.y, 64, 64 }, valign = "top", halign = "right" },
					{ atlas = true, h = 2/3, w = 2/3, texture_rect = { icons.doc_bag.x, icons.doc_bag.y, 64, 64 }, valign = "bottom", halign = "left" },
				})
			end
			if SHOW_MINIONS then
				list:register_item("minions", "HorizontalList", { align = "top", w = list_width, h = 50, left_to_right = true, item_margin = 5 })
				list:item("minions"):set_static_item("LeftListIcon", 1, 4/5, { 
					{ atlas = true, texture_rect = { 6 * 64, 8 * 64, 64, 64 } },
				})
			end
			if SHOW_PAGERS then
				list:register_item("pagers", "HorizontalList", { align = "top", w = list_width, h = 40, left_to_right = true, item_margin = 5 })
				list:item("pagers"):set_static_item("LeftListIcon", 1, 1, { 
					{ waypoints = true, texture_rect = { 64, 0, 32, 32 } },
				})
			end
		end
		
		function HUDManager:_setup_right_list()
			local list_width = 800
			local list_height = 800
			local x = self:script(PlayerBase.PLAYER_INFO_HUD_PD2).panel:right() - list_width
			local y = 0
			
			local list = managers.hud:register_hud_list("right_side_list", "VerticalList", { align = "right", x = x, y = y, w = list_width, h = list_height, top_to_bottom = true, item_margin = 5 })
			
			if SHOW_ECMS then
				local ecm_list = list:register_item("ecm_list", "HorizontalList", { align = "top", w = list_width, h = 25, left_to_right = false, item_margin = 3 })
				ecm_list:set_static_item("ECMIcon")
			end
			
			if SHOW_ENEMIES then
				local unit_types = {
					cop = "EnemyCountItem",
					security = "GuardCountItem",
					thug = "ThugCountItem",
					sniper = "SniperCountItem",
					tank = "DozerCountItem",
					taser = "TaserCountItem",
					spooc = "CloakerCountItem",
					shield = "ShieldCountItem",
					mobster_boss = "MobsterBossCountItem",
				}
				local unit_count_list = list:register_item("enemy_count_list", "HorizontalList", { align = "top", w = list_width, h = 50, left_to_right = false, item_margin = 3 })
				for name, item in pairs(unit_types) do
					unit_count_list:register_item(name, item)
				end
			end
			
			if SHOW_CIVILIANS or SHOW_HOSTAGES then
				local hostage_count_list = list:register_item("hostage_count_list", "HorizontalList", { align = "top", w = list_width, h = 50, left_to_right = false, item_margin = 3 })
				
				if SHOW_HOSTAGES then
					hostage_count_list:register_item("CopHostages", "CopHostageCountItem")
					hostage_count_list:register_item("CivHostages", "CivHostageCountItem")
				end
				if SHOW_CIVILIANS then
					hostage_count_list:register_item("CivCount", "CivilianCountItem")
				end
			end
			
			if SHOW_LOOT then
				local loot_list = list:register_item("loot_list", "HorizontalList", { align = "top", x = 0, y = 0, w = 800, h = AGGREGATE_LOOT and 25 or 50, left_to_right = false, item_margin = 3 })
				if AGGREGATE_LOOT then
					loot_list:register_item("loot", "TotalLootItem")
				else
					for id, text in pairs(ObjectInteractionManager.LOOT_TYPES) do
						loot_list:register_item(id, "LootItem", id, text)
					end
				end
			end
		end
		
		function HUDManager:update_unit_count(utype, count)
			if self._lists_initialized then
				if utype == "civilian" then
					self:get_hud_list("right_side_list"):item("hostage_count_list"):item("CivCount"):set_count(count - managers.groupai:state():civilian_hostage_count())
				else
					self:get_hud_list("right_side_list"):item("enemy_count_list"):item(utype):set_count(count)
				end
			end
		end
		
		function HUDManager:_setup_buff_list()
			local function register_buff_item(buff)
				if not buff.ignore then
					local item = managers.hud:get_hud_list("buff_list"):item(buff.type):register_item(buff.name, buff.class or "BuffItemBase", buff)
					
					if item and buff.no_fade then
						item:set_fade_duration(0)
					end
					
					return item
				end
			end
		
			local list_width = 800
			local list_height = 150
			local x = self:script(PlayerBase.PLAYER_INFO_HUD_PD2).panel:right() - list_width
			local y = self:script(PlayerBase.PLAYER_INFO_HUD_PD2).panel:bottom() - (125 + list_height)
			
			local buff_list = self:register_hud_list("buff_list", "VerticalList", { align = "right", x = x, y = y , w = list_width, h = list_height, top_to_bottom = false, item_margin = 0 })
			buff_list:register_item("debuff", "HorizontalList", { align = "bottom", w = buff_list:panel():w(), h = buff_list:panel():h() / 3, left_to_right = false, item_margin = 0 })
			buff_list:register_item("team", "HorizontalList", { align = "bottom", w = buff_list:panel():w(), h = buff_list:panel():h() / 3, left_to_right = false, item_margin = 0 })
			buff_list:register_item("temporary", "HorizontalList", { align = "bottom", w = buff_list:panel():w(), h = buff_list:panel():h() / 3, left_to_right = false, item_margin = 0 })
			
			for _, buff in pairs(DynamicHUDList.BuffItemBase.BUFFS.temporary) do
				if not buff.ignore then
					local new_item = register_buff_item(buff)
					if new_item and buff.has_aced and buff.has_aced() then
						new_item:set_aced(true)
					end
				end
			end
			
			for category, upgrade_data in pairs(DynamicHUDList.BuffItemBase.BUFFS.team) do
				for upgrade, buff in pairs(upgrade_data) do
					if not buff.ignore then
						local new_item = register_buff_item(buff)
						
						--Manually activate local player team buffs
						if new_item and managers.player:has_team_category_upgrade(category, upgrade) then
							if buff.level then
								new_item:set_level(buff.level)
							end
							
							if buff.has_aced and buff.has_aced() then
								new_item:set_aced(true)
							end
							
							new_item:activate()
						end
					end
				end
			end
			
			for _, buff in pairs(DynamicHUDList.BuffItemBase.BUFFS.special) do
				if not buff.ignore then
					local new_item = register_buff_item(buff)
					if new_item and buff.has_aced and buff.has_aced() then
						new_item:set_aced(true)
					end
				end
			end
		end
		
		function HUDManager:update_buff_item(buff, callbacks)
			local item = not buff.ignore and self:get_hud_list("buff_list"):item(buff.type):item(buff.name) or nil
			
			if item then
				for cbk, args in pairs(callbacks or {}) do
					item[cbk](item, (type(args) == "table") and unpack(args) or args)
				end
			end
		end
		
		------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

		DynamicHUDList = DynamicHUDList or {}
		DynamicHUDList.NodeBase = DynamicHUDList.NodeBase or class() --DO NOT INSTANTIATE THIS CLASS
		function DynamicHUDList.NodeBase:init(parent, name, params)
			self._parent = parent
			self._name = name
			self._align = params.align or "center"
			self._fade_duration = 0.25
			
			self._panel = (parent and parent:panel() or managers.hud:script(PlayerBase.PLAYER_INFO_HUD_PD2).panel):panel({
				name = name,
				visible = false,
				alpha = 0,
				w = params.w or 0,
				h = params.h or 0,
				x = params.x or 0,
				y = params.y or 0,
				layer = 10
			})
		end

		function DynamicHUDList.NodeBase:post_init(...) 
		
		end

		function DynamicHUDList.NodeBase:name()
			return self._name
		end

		function DynamicHUDList.NodeBase:panel()
			return self._panel
		end

		function DynamicHUDList.NodeBase:parent()
			return self._parent
		end

		function DynamicHUDList.NodeBase:align()
			return self._align
		end

		function DynamicHUDList.NodeBase:is_visible()
			return self._panel:visible()
		end

		function DynamicHUDList.NodeBase:is_active()
			return self._active
		end

		function DynamicHUDList.NodeBase:is_deleted()
			return self._scheduled_for_deletion
		end

		function DynamicHUDList.NodeBase:is_animating()
			return (self._move_co or self._fade_co) and true or false
		end

		function DynamicHUDList.NodeBase:is_list()
			return self._is_list
		end

		function DynamicHUDList.NodeBase:set_fade_duration(duration)
			self._fade_duration = duration or self._fade_duration
		end

		function DynamicHUDList.NodeBase:activate()
			if not self:is_active() then
				self._active = true
				self:_show()
			end
		end

		function DynamicHUDList.NodeBase:deactivate()
			if self:is_active() then
				self._active = false
				self:_hide()
			end
		end

		function DynamicHUDList.NodeBase:set_active(status)
			if status then
				self:activate()
			else
				self:deactivate()
			end
		end

		function DynamicHUDList.NodeBase:delete(instant)
			if alive(self._panel) then
				self:deactivate()
				self:_hide(instant)
				self._scheduled_for_deletion = true
			end
		end

		function DynamicHUDList.NodeBase:_show(instant)
			if alive(self._panel) then
				if self._parent then
					self._parent:cbk_item_visible(self._name, self:is_visible())
				end
				
				local init = self._panel:alpha()
				local final = 1
				local duration = instant and 0 or self._fade_duration * (1 - init/final)
				self:_set_fade(init, final, duration)
				self:panel():show()
			end
		end

		function DynamicHUDList.NodeBase:_hide(instant)
			if alive(self._panel) then
				if self._parent then
					self._parent:cbk_hide_item(self._name, self:is_visible())
				end
				
				local init = self._panel:alpha()
				local final = 0
				local duration = instant and 0 or self._fade_duration * init
				self:_set_fade(init, final, duration)
			end
		end

		function DynamicHUDList.NodeBase:move(x, y, duration)
			duration = duration or 0.25
			local init_x = self:panel():x()
			local init_y = self:panel():y()
			
			--if init_x ~= x or init_y ~= y then
				self._move_co = coroutine.create(DynamicHUDList.NodeBase._animate_move)
				coroutine.resume(self._move_co, self, init_x, init_y, x, y, duration)
			--end
		end

		function DynamicHUDList.NodeBase:_set_fade(init, final, duration)
			duration = duration or 0.5
			--if init ~= final then
				self._fade_co = coroutine.create(DynamicHUDList.NodeBase._animate_fade)
				coroutine.resume(self._fade_co, self, init, final, duration)
			--end
		end

		function DynamicHUDList.NodeBase:animate(t, dt)
			if self._move_co then
				coroutine.resume(self._move_co, dt)
			end
			if self._fade_co then
				coroutine.resume(self._fade_co, dt)
			end
		end

		function DynamicHUDList.NodeBase:_animate_fade(init, final, duration)
			local t = duration
			
			while t > 0 do
				local dt = coroutine.yield()
				t = math.max(t - dt, 0)
				self._panel:set_alpha(t/duration * (init-final) + final)
			end
			
			if final == 0 then
				self._panel:hide()
				if self._parent then
					self._parent:cbk_item_hidden(self._name)
				end
			end

			self._panel:set_alpha(final)
			self._fade_co = nil
		end

		function DynamicHUDList.NodeBase:_animate_move(init_x, init_y, x, y, duration)
			local t = duration
			
			while t > 0 do
				local dt = coroutine.yield()
				t = math.max(t - dt, 0)
				self._panel:set_x(t/duration * (init_x-x) + x)
				self._panel:set_y(t/duration * (init_y-y) + y)
			end
			
			self._panel:set_x(x)
			self._panel:set_y(y)
			self._move_co = nil
		end

		------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

		DynamicHUDList.ItemBase = DynamicHUDList.ItemBase or class(DynamicHUDList.NodeBase) --DO NOT INSTANTIATE THIS CLASS
		DynamicHUDList.ItemBase.DEFAULT_COLOR_TABLE = {
			{ ratio = 0.0, color = Color(1, 0.9, 0.1, 0.1) }, --Red
			{ ratio = 0.5, color = Color(1, 0.9, 0.9, 0.1) }, --Yellow
			{ ratio = 1.0, color = Color(1, 0.1, 0.9, 0.1) } --Green
		}
		function DynamicHUDList.ItemBase:init(parent, name, params)
			params.w = params.w or parent:default_item_w()
			params.h = params.h or parent:default_item_h()
			DynamicHUDList.NodeBase.init(self, parent, name, params)
			
			self._is_list = false
		end

		function DynamicHUDList.ItemBase:_get_color_from_table(value, max_value, color_table, default_color)
			local color_table = color_table or DynamicHUDList.ItemBase.DEFAULT_COLOR_TABLE
			local ratio = math.clamp(value / max_value, 0 , 1)
			local tmp_color = color_table[#color_table].color
			local color = default_color or Color(tmp_color.alpha, tmp_color.red, tmp_color.green, tmp_color.blue)
			
			for i, data in ipairs(color_table) do
				if ratio < data.ratio then
					local nxt = color_table[math.clamp(i-1, 1, #color_table)]
					local scale = (ratio - data.ratio) / (nxt.ratio - data.ratio)
					color = Color(
						(data.color.alpha or 1) * (1-scale) + (nxt.color.alpha or 1) * scale, 
						(data.color.red or 0) * (1-scale) + (nxt.color.red or 0) * scale, 
						(data.color.green or 0) * (1-scale) + (nxt.color.green or 0) * scale, 
						(data.color.blue or 0) * (1-scale) + (nxt.color.blue or 0) * scale)
					break
				end
			end
			
			return color
		end

		------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

		DynamicHUDList.ListBase = DynamicHUDList.ListBase or class(DynamicHUDList.NodeBase) --DO NOT INSTANTIATE THIS CLASS
		function DynamicHUDList.ListBase:init(parent, name, params)
			DynamicHUDList.ListBase.super.init(self, parent, name, params)

			self._item_fade_duration = params.item_fade_duration or 0.25
			self._item_move_duration = params.item_move_duration or 0.25
			self._item_margin = params.item_margin or 0
			self._margin = params.item_margin or 0
			self._stack = params.stack or false
			self._items = {}
			self._shown_items = {}
			self._is_list = true
		end

		function DynamicHUDList.ListBase:item(name)
			return self._items[name]
		end

		function DynamicHUDList.ListBase:items()
			return table.size(self._items)
		end

		function DynamicHUDList.ListBase:active_items()
			local count  = 0
			for name, item in pairs(self._items) do
				if item:is_active() then
					count = count + 1
				end
			end
			return count
		end

		function DynamicHUDList.ListBase:shown_items()
			return #self._shown_items
		end

		function DynamicHUDList.ListBase:update(t, dt)
			local delete_items = {}
			for name, item in pairs(self._items) do
				if item.update and (item:is_active() or item:is_list()) then
					item:update(t, dt)
				end
				
				if item:is_animating() then
					item:animate(t, dt)
				end
				
				if not item:is_visible() and item:is_deleted() then
					table.insert(delete_items, name)
				end
				
				--[[
				if item:is_visible() then
					if item:is_animating() then
						item:animate(t, dt)
					end
				elseif item:is_deleted() then
					table.insert(delete_items, name)
				end
				]]
			end
			
			for _, name in ipairs(delete_items) do
				self._panel:remove(self._panel:child(name))
				self._items[name] = nil
			end
		end

		function DynamicHUDList.ListBase:register_item(name, class, ...)
			if not self._items[name] then
				self._items[name] = _G.DynamicHUDList[class] and _G.DynamicHUDList[class]:new(self, name, ...)
				
				if self._items[name] then
					self._items[name]:set_fade_duration(self._item_fade_duration)
					self:_set_default_item_position(name)
					self._items[name]:post_init(...)
				end
			end
			
			return self._items[name]
		end

		function DynamicHUDList.ListBase:unregister_item(name, instant)
			local item = self._items[name]
			if item then
				item:delete(instant)
			end
		end

		function DynamicHUDList.ListBase:set_static_item(class, ...)
			self:delete_static_item()
			
			self._static_item = _G.DynamicHUDList[class] and _G.DynamicHUDList[class]:new(self, "static_list_item", ...)
			if self._static_item then
				self:setup_static_item()
				self._static_item:panel():show()
				self._static_item:panel():set_alpha(1)
			end
			
			return self._static_item
		end

		function DynamicHUDList.ListBase:delete_static_item()
			if self._static_item then
				self._static_item:delete(true)
				self._static_item = nil
			end
		end

		function DynamicHUDList.ListBase:cbk_item_hidden(name)
			for i, shown_item in ipairs(self._shown_items) do
				if shown_item == name then
					table.remove(self._shown_items, i)
					self:_update_item_positions()
					break
				end
			end
			
			if #self._shown_items <= 0 then
				self:deactivate()
			end
		end

		function DynamicHUDList.ListBase:cbk_item_visible(name, was_visible)
			self:activate()
			if not was_visible then
				self:_set_item_position(name)
				table.insert(self._shown_items, name)
				--table.insert(self._shown_items, name, 1 + (self._stack and 0 or #self._shown_items))
				self:_update_item_positions()
			end
		end

		function DynamicHUDList.ListBase:cbk_hide_item(name, was_visible)
		--[[
			for i, shown_item in ipairs(self._shown_items) do
				if shown_item == name then
					table.remove(self._shown_items, i)
					self:_update_item_positions()
					break
				end
			end
		]]
			if #self._shown_items <= 1 then
				self:deactivate()
			end
		end

		------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

		DynamicHUDList.HorizontalList = DynamicHUDList.HorizontalList or class(DynamicHUDList.ListBase)
		function DynamicHUDList.HorizontalList:init(parent, name, params)
			params.align = params.align == "top" and "top" or params.align == "bottom" and "bottom" or "center"
			DynamicHUDList.HorizontalList.super.init(self, parent, name, params)
			self._left_to_right = params.left_to_right
		end

		function DynamicHUDList.HorizontalList:default_item_w()
			return self._panel:w() / 10
		end

		function DynamicHUDList.HorizontalList:default_item_h()
			return self._panel:h()
		end

		function DynamicHUDList.HorizontalList:_set_default_item_position(name)
			local item = self._items[name]
			local offset = self._panel:h() - item:panel():h()
			local y = item:align() == "top" and 0 or item:align() == "bottom" and offset or offset / 2
			item:panel():set_top(y)
		end

		function DynamicHUDList.HorizontalList:setup_static_item()
			local item = self._static_item
			local offset = self._panel:h() - item:panel():h()
			local y = item:align() == "top" and 0 or item:align() == "bottom" and offset or offset / 2
			local x = self._left_to_right and 0 or self._panel:w() - item:panel():w()
			item:panel():set_left(x)
			item:panel():set_top(y)
			self:_update_item_positions()
		end

		function DynamicHUDList.HorizontalList:_set_item_position(name)
			local item = self._items[name]
			if item then
				local last_item = #self._shown_items > 0 and self._items[self._shown_items[#self._shown_items] ]
				local init_x = 0
				local static_w = self._static_item and self._static_item:panel():w() or 0

				if self._left_to_right then
					init_x = (last_item and last_item:panel():x() + last_item:panel():w() or static_w) + self._item_margin
				else
					init_x = (last_item and last_item:panel():x() or ((self._panel:w() - static_w))) - (self._item_margin + item:panel():w())
				end
				
				item:panel():set_x(init_x)
				
			end
		end

		function DynamicHUDList.HorizontalList:_update_item_positions()
			local prev_width = self._static_item and (self._static_item:panel():w() + self._item_margin) or 0
			for i, item_name in ipairs(self._shown_items) do
				local item = self._items[item_name]
				if item then
					local width = item:panel():w()
					local new_x = (self._left_to_right and prev_width) or (self._panel:w() - (width+prev_width))
					item:move(new_x, item:panel():y(), self._item_move_duration)
					prev_width = prev_width + width + self._item_margin
				end
			end
		end

		------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

		DynamicHUDList.VerticalList = DynamicHUDList.VerticalList or class(DynamicHUDList.ListBase)
		function DynamicHUDList.VerticalList:init(parent, name, params)
			params.align = params.align == "left" and "left" or params.align == "right" and "right" or "center"
			DynamicHUDList.VerticalList.super.init(self, parent, name, params)
			self._top_to_bottom = params.top_to_bottom
		end

		function DynamicHUDList.VerticalList:default_item_w()
			return self._panel:w()
		end

		function DynamicHUDList.VerticalList:default_item_h()
			return self._panel:h() / 10
		end

		function DynamicHUDList.VerticalList:_set_default_item_position(name)
			local item = self._items[name]
			local offset = self._panel:w() - item:panel():w()
			local x = item:align() == "left" and 0 or item:align() == "right" and offset or offset / 2
			item:panel():set_left(x)
		end

		function DynamicHUDList.VerticalList:setup_static_item()
			local item = self._static_item
			local offset = self._panel:w() - item:panel():w()
			local x = item:align() == "left" and 0 or item:align() == "right" and offset or offset / 2
			local y = self._top_to_bottom and 0 or self._panel:h() - item:panel():h()
			item:panel():set_left(x)
			item:panel():set_y(y)
			self:_update_item_positions()
		end

		function DynamicHUDList.VerticalList:_set_item_position(name)
			local item = self._items[name]
			if item then
				local last_item = #self._shown_items > 0 and self._items[self._shown_items[#self._shown_items] ]
				local init_y = 0
				local static_h = self._static_item and self._static_item:panel():h() or 0

				if self._top_to_bottom then
					init_y = (last_item and last_item:panel():y() + last_item:panel():h() or static_h) + self._item_margin
				else
					init_y = (last_item and last_item:panel():y() or ((self._panel:h() - static_h))) - (self._item_margin + item:panel():h())
				end

				item:panel():set_y(init_y)
			end
		end

		function DynamicHUDList.VerticalList:_update_item_positions()
			local prev_height = self._static_item and (self._static_item:panel():h() + self._item_margin) or 0
			for i, item_name in ipairs(self._shown_items) do
				local item = self._items[item_name]
				if item then
					local height = item:panel():h()
					local new_y = (self._top_to_bottom and prev_height) or (self._panel:h() - (height+prev_height))
					item:move(item:panel():x(), new_y, self._item_move_duration)
					prev_height = prev_height + height + self._item_margin
				end
			end
		end

		------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

		DynamicHUDList.UnitCountItem = DynamicHUDList.UnitCountItem or class(DynamicHUDList.ItemBase)
		function DynamicHUDList.UnitCountItem:init(parent, name, icon, color)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "right", w = parent:panel():h() / 2, h = parent:panel():h() })
			
			self._count = 0
			
			self._box = HUDBGBox_create(self._panel, {
					w = self._panel:w(),
					h = self._panel:w(),
				}, {})
			self._box:set_bottom(self._panel:bottom())
			
			self._text = self._box:text({
				name = "text",
				text = "0",
				align = "center",
				vertical = "center",
				w = self._box:w(),
				h = self._box:h(),
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._box:h() * 0.6
			})
			
			if icon then
				self._icon = self._panel:bitmap({
					name = "icon",
					texture = icon.texture or "guis/textures/pd2/skilltree/icons_atlas",
					texture_rect = icon.atlas and { icon.x * 64, icon.y * 64, 64, 64 } or nil,
					h = self._panel:w(),
					w = self._panel:w(),
					blend_mode = "add",
					color = color or Color.white,
				})
				self._icon:set_top(self._panel:top())
			end
		end
		
		function DynamicHUDList.UnitCountItem:post_init()
			if self._unit_type then
				self:set_count(managers.enemy and managers.enemy:get_unit_count(self._unit_type) or 0)
			end
		end
		
		function DynamicHUDList.UnitCountItem:set_count(num)
			self._count = num
			self._text:set_text(tostring(self._count))
			if self._count > 0 then
				self:activate()
			else
				self:deactivate()
			end
		end
		
		function DynamicHUDList.UnitCountItem:change_count(diff)
			self:set_count(self._count + diff)
		end
		
		function DynamicHUDList.UnitCountItem:get_count()
			return self._count
		end
		
		DynamicHUDList.EnemyCountItem = DynamicHUDList.EnemyCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.EnemyCountItem:init(parent, name)
			self._unit_type = "cop"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 0, y = 5}, Color.red)
		end
		
		DynamicHUDList.GuardCountItem = DynamicHUDList.GuardCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.GuardCountItem:init(parent, name)
			self._unit_type = "security"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 0, y = 5}, Color.yellow)
		end
		
		DynamicHUDList.ThugCountItem = DynamicHUDList.ThugCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.ThugCountItem:init(parent, name)
			self._unit_type = "thug"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 0, y = 5}, Color(1, 0.64, 0))
		end
		
		DynamicHUDList.SniperCountItem = DynamicHUDList.SniperCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.SniperCountItem:init(parent, name)
			self._unit_type = "sniper"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 6, y = 5}, Color.red)
		end
		
		DynamicHUDList.DozerCountItem = DynamicHUDList.DozerCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.DozerCountItem:init(parent, name)
			self._unit_type = "tank"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 3, y = 1}, Color.red)
		end
		
		DynamicHUDList.TaserCountItem = DynamicHUDList.TaserCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.TaserCountItem:init(parent, name)
			self._unit_type = "taser"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 3, y = 5}, Color.red)
		end
		
		DynamicHUDList.CloakerCountItem = DynamicHUDList.CloakerCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.CloakerCountItem:init(parent, name)
			self._unit_type = "spooc"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 1, y = 3}, Color.red)
		end
		
		DynamicHUDList.MobsterBossCountItem = DynamicHUDList.MobsterBossCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.MobsterBossCountItem:init(parent, name)
			self._unit_type = "mobster_boss"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 3, y = 1}, Color.yellow)
		end
		
		DynamicHUDList.ShieldCountItem = DynamicHUDList.ShieldCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.ShieldCountItem:init(parent, name)
			self._unit_type = "shield"
			DynamicHUDList.UnitCountItem.init(self, parent, name)
			
			--Special icon composite, ignore regular super class icon setup
			self._icon = self._panel:bitmap({
				name = "icon",
				texture = "guis/textures/pd2/skilltree/icons_atlas",
				texture_rect = { 64, 64, 64, 64 },
				h = self._panel:w(),
				w = self._panel:w() * 0.8,
				blend_mode = "add",
				color = Color.red,
			})
			self._icon:set_top(self._panel:top())
			self._icon:set_right(self._panel:right() - self._icon:w() * 0.2)
				
			self._icon_extra = self._panel:bitmap({
				name = "icon_extra",
				texture = "guis/textures/pd2/skilltree/icons_atlas",
				texture_rect = { 2 * 64, 0, 64 * 0.3, 64 },
				rotation = 180,
				h = self._panel:w(),
				w = self._panel:w() * 0.4,
				blend_mode = "add",
				color = Color.red,
			})
			self._icon_extra:set_right(self._panel:right())
			self._icon_extra:set_top(self._panel:top())
		end
		
		DynamicHUDList.CivHostageCountItem = DynamicHUDList.CivHostageCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.CivHostageCountItem:init(parent, name)
			DynamicHUDList.UnitCountItem.init(self, parent, name, { texture = "guis/textures/pd2/hud_icon_hostage" })
		end
		
		DynamicHUDList.CivilianCountItem = DynamicHUDList.CivilianCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.CivilianCountItem:init(parent, name)
			self._unit_type = "civilian"
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 6, y = 7})
		end
		
		function DynamicHUDList.CivilianCountItem:set_count(num)
			self._count = num
			self._text:set_text(tostring(self._count))
			if self._count > 0 then
				self:activate()
			else
				self:deactivate()
			end
		end
		
		DynamicHUDList.CopHostageCountItem = DynamicHUDList.CopHostageCountItem or class(DynamicHUDList.UnitCountItem)
		function DynamicHUDList.CopHostageCountItem:init(parent, name)
			DynamicHUDList.UnitCountItem.init(self, parent, name, { atlas = true, x = 2, y = 8})
		end

		function DynamicHUDList.CopHostageCountItem:set_free_slots(status)
			self._text:set_color(status and Color.white or Color.red)
		end	
		
		DynamicHUDList.ECMIcon = DynamicHUDList.ECMIcon or class(DynamicHUDList.ItemBase)
		function DynamicHUDList.ECMIcon:init(parent, name)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "center", w = parent:panel():h(), h = parent:panel():h() })
			
			self._icon = self._panel:bitmap({
				name = "icon",
				texture = "guis/textures/pd2/skilltree/icons_atlas",
				texture_rect = { 1 * 64, 4 * 64, 64, 64 },
				h = self:panel():h(),
				w = self:panel():w(),
				blend_mode = "add",
				color = Color.white,
			})
		end
		
		DynamicHUDList.ECMItem = DynamicHUDList.ECMItem or class(DynamicHUDList.ItemBase)
		function DynamicHUDList.ECMItem:init(parent, name)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "right", w = parent:panel():h(), h = parent:panel():h() })
			
			self._max_duration = tweak_data.upgrades.ecm_jammer_base_battery_life * 
				tweak_data.upgrades.values.ecm_jammer.duration_multiplier[1] * 
				tweak_data.upgrades.values.ecm_jammer.duration_multiplier_2[1]
			
			self._box = HUDBGBox_create(self._panel, {
					w = self._panel:w(),
					h = self._panel:h(),
				}, {})
			
			self._text = self._box:text({
				name = "text",
				align = "center",
				vertical = "center",
				w = self._box:w(),
				h = self._box:h(),
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._box:h() * 0.6,
			})
		end
		
		function DynamicHUDList.ECMItem:update_timer(t, time_left)
			local remaining = math.max(time_left, 0)
			
			self._text:set_text(string.format("%.1f", remaining))
			self._text:set_color(self:_get_color_from_table(remaining, self._max_duration))
			if remaining <= 0 then
				self:delete()
			end
		end

		
		DynamicHUDList.TotalLootItem = DynamicHUDList.TotalLootItem or class(DynamicHUDList.ItemBase)
		function DynamicHUDList.TotalLootItem:init(parent, name, loot_id, loot_text)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "right", w = parent:panel():h() * 2, h = parent:panel():h() })
		
			self._box = HUDBGBox_create(self._panel, {
					w = self._panel:h(),
					h = self._panel:h(),
				}, {})
			self._box:set_left(self._panel:left())
			
			self._text = self._box:text({
				name = "text",
				align = "center",
				vertical = "center",
				w = self._box:w(),
				h = self._box:h(),
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._box:h() * 0.6
			})
			
			self._icon = self._panel:bitmap({
				name = "icon",
				texture = "guis/textures/pd2/hud_tabs",
				texture_rect = { 32, 32, 32, 32 },
				h = self._panel:h(),
				w = self._panel:h(),
				blend_mode = "add",
				alpha = 0.75,
				layer = 1
			})
			self._icon:set_right(self._panel:right())

			self._count = 0
		end
		
	--	function DynamicHUDList.TotalLootItem:post_init()
	--		self:set_count(managers.interaction and managers.interaction:loot_count() or 0)
	--	end
		
		function DynamicHUDList.TotalLootItem:set_count(value, types)
			if self._count > 0 and value > 0 then
				self._text:stop()
				self._text:animate(callback(self, self, "_animate_change"), 1, self._count < value)
			end
			
			self._count = value
			self._text:set_text(self._count)
			if LOOT_DEBUG then
				if self._count ~= 0 then
					self:activate()
				else
					self:deactivate()
				end
			else
				if self._count > 0 then
					self:activate()
				else
					self:deactivate()
				end
			end
		end
		
		function DynamicHUDList.TotalLootItem:change_count(diff)
			self:set_count(self._count + diff)
		end
		
		function DynamicHUDList.TotalLootItem:_animate_change(text, duration, incr)
			text:set_color(Color.white)
			
			local t = duration
			while t > 0 do
				local dt = coroutine.yield()
				t = math.max(t - dt, 0)
				local ratio = math.sin(t/duration * 1440 + 90) * 0.5 + 0.5
				text:set_color(Color(incr and ratio or 1, incr and 1 or ratio, ratio))
			end
			
			text:set_color(Color.white)
		end
		
		
		DynamicHUDList.LootItem = DynamicHUDList.LootItem or class(DynamicHUDList.ItemBase)
		function DynamicHUDList.LootItem:init(parent, name, loot_id, loot_text)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "right", w = parent:panel():h() / 2, h = parent:panel():h() })
		
			self._box = HUDBGBox_create(self._panel, {
					w = self._panel:w(),
					h = self._panel:w(),
				}, {})
			self._box:set_bottom(self._panel:bottom())
			
			self._text = self._box:text({
				name = "text",
				align = "center",
				vertical = "center",
				w = self._box:w(),
				h = self._box:h(),
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._box:h() * 0.6
			})
			
			self._icon = self._panel:bitmap({
				name = "icon",
				texture = "guis/textures/pd2/hud_tabs",
				texture_rect = { 32, 32, 32, 32 },
				h = self._panel:w(),
				w = self._panel:w() * 1.2,
				blend_mode = "add",
				alpha = 0.75,
				layer = 1
			})
			self._icon:set_center(self._panel:center())
			self._icon:set_top(self._panel:top())
			
			self._name_text = self._panel:text({
				name = "text",
				text = string.sub(loot_text, 1, 5),
				align = "center",
				vertical = "center",
				w = self._panel:w(),
				h = self._panel:w(),
				color = Color(0.0, 0.5, 0.0),
				blend_mode = "normal",
				font = tweak_data.hud_corner.assault_font,
				font_size = self._panel:w() * 0.4,
				layer = 10
			})
			self._name_text:set_center(self._icon:center())
			self._name_text:set_y(self._name_text:y() + self._icon:h() * 0.1)
			
			self._loot_id = loot_id
			self._count = 0
		end
		
		function DynamicHUDList.LootItem:post_init()
			self:set_count(managers.interaction and managers.interaction:loot_count(self._loot_id) or 0)
		end
		
		DynamicHUDList.LootItem.set_count = DynamicHUDList.TotalLootItem.set_count
		DynamicHUDList.LootItem.change_count = DynamicHUDList.TotalLootItem.change_count
		DynamicHUDList.LootItem._animate_change = DynamicHUDList.TotalLootItem._animate_change
		
		------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

		DynamicHUDList.LeftListIcon = DynamicHUDList.LeftListIcon or class(DynamicHUDList.ItemBase)
		function DynamicHUDList.LeftListIcon:init(parent, name, ratio_w, ratio_h, icons)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "center", w = parent:panel():h() * (ratio_w or 1), h = parent:panel():h() * (ratio_h or 1) })
			
			self._icons = {}
			for i, icon in ipairs(icons) do
				local bitmap = self._panel:bitmap({
					name = "icon_" .. tostring(i),
					texture = icon.atlas and "guis/textures/pd2/skilltree/icons_atlas" or icon.waypoints and "guis/textures/pd2/pd2_waypoints" or icon.texture,
					texture_rect = icon.texture_rect or nil,
					h = self:panel():w() * (icon.h or 1),
					w = self:panel():w() * (icon.w or 1),
					blend_mode = "add"
				})
				
				bitmap:set_center(self._panel:center())
				if icon.valign == "top" then 
					bitmap:set_top(self._panel:top())
				elseif icon.valign == "bottom" then 
					bitmap:set_bottom(self._panel:bottom())
				end
				if icon.halign == "left" then
					bitmap:set_left(self._panel:left())
				elseif icon.halign == "right" then
					bitmap:set_right(self._panel:right())
				end
			
				table.insert(self._icons, bitmap)
			end
		end

		DynamicHUDList.TimerItem = DynamicHUDList.TimerItem or class(DynamicHUDList.ItemBase)
		DynamicHUDList.TimerItem.STANDARD_COLOR = Color(1, 1, 1, 1)
		DynamicHUDList.TimerItem.UPGRADE_COLOR = Color(1, 0.0, 0.8, 1.0)
		DynamicHUDList.TimerItem.DISABLED_COLOR = Color(1, 1, 0, 0)
		DynamicHUDList.TimerItem.FLASH_SPEED = 2
		function DynamicHUDList.TimerItem:init(parent, name, position, text)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "left", w = parent:panel():h() * 4/5, h = parent:panel():h() })
			
			self._flash_color_table = {
				{ ratio = 0.0, color = self.DISABLED_COLOR },
				{ ratio = 1.0, color = self.STANDARD_COLOR }
			}
			self._current_color = self.STANDARD_COLOR
			self._position = position
			
			self._type_text = self._panel:text({
				name = "type_text",
				text = text,
				align = "center",
				vertical = "top",
				w = self._panel:w(),
				h = self._panel:h() * 0.3,
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._panel:h() * 1/3
			})
			
			self._box = HUDBGBox_create(self._panel, {
					w = self._panel:w(),
					h = self._panel:h() * 0.7,
				}, {})
			self._box:set_bottom(self._panel:bottom())
			
			self._distance_text = self._box:text({
				name = "distance",
				align = "center",
				vertical = "top",
				w = self._box:w(),
				h = self._box:h(),
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._box:h() * 0.4
			})
			
			self._time_text = self._box:text({
				name = "time",
				align = "center",
				vertical = "bottom",
				w = self._box:w(),
				h = self._box:h(),
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._box:h() * 0.6
			})
			
			self:_set_colors(self._current_color)
		end

		function DynamicHUDList.TimerItem:update_timer(t, time_left)
			local distance = 0
			if alive(managers.player:player_unit()) then
				distance = mvector3.normalize(managers.player:player_unit():position() - self._position) / 100
			end
			
			self._time_text:set_text(string.format("%d:%02d", math.floor(time_left/60), time_left%60))
			self._distance_text:set_text(string.format("%.0fm", distance))
			
			if self._jammed then
				local new_color = self:_get_color_from_table(math.sin(t*360 * self.FLASH_SPEED) * 0.5 + 0.5, 1, self._flash_color_table, self.STANDARD_COLOR)
				self:_set_colors(new_color)
			end
		end
		
		function DynamicHUDList.TimerItem:set_jammed(status)
			if not status then
				self:_set_colors(self._current_color)
			end
			self._jammed = status
		end
		
		function DynamicHUDList.TimerItem:set_powered(status)
			self:set_jammed(status)
		end

		function DynamicHUDList.TimerItem:_set_colors(color)
			self._time_text:set_color(color)
			self._type_text:set_color(color)
			self._distance_text:set_color(color)
		end
		
		DynamicHUDList.DrillItem = DynamicHUDList.DrillItem or class(DynamicHUDList.TimerItem)
		function DynamicHUDList.DrillItem:init(parent, name, unit)
			DynamicHUDList.TimerItem.init(self, parent, name, unit:position(), "Drill")
		end
		
		function DynamicHUDList.DrillItem:set_can_upgrade(status)
			self._can_upgrade = status
			self._current_color = status and self.UPGRADE_COLOR or self.STANDARD_COLOR
			self._flash_color_table[2].color = status and self.UPGRADE_COLOR or self.STANDARD_COLOR
			self:_set_colors(self._current_color)
		end
		
		DynamicHUDList.HackItem = DynamicHUDList.HackItem or class(DynamicHUDList.TimerItem)
		function DynamicHUDList.HackItem:init(parent, name, unit)
			DynamicHUDList.TimerItem.init(self, parent, name, unit:position(), "Hack")
		end
		
		DynamicHUDList.SawItem = DynamicHUDList.SawItem or class(DynamicHUDList.TimerItem)
		function DynamicHUDList.SawItem:init(parent, name, unit)
			DynamicHUDList.TimerItem.init(self, parent, name, unit:position(), "Saw")
		end
		
		DynamicHUDList.DigitalTimerItem = DynamicHUDList.DigitalTimerItem or class(DynamicHUDList.TimerItem)
		function DynamicHUDList.DigitalTimerItem:init(parent, name, unit)
			DynamicHUDList.TimerItem.init(self, parent, name, unit:position(), "Timer")
		end

		DynamicHUDList.EquipmentItem = DynamicHUDList.EquipmentItem or class(DynamicHUDList.ItemBase)
		DynamicHUDList.EquipmentItem.ICON_TABLE = {
			sentry = { x = 7 * 64, y = 5 * 64 },
			ammo_bag = { x = 1 * 64, y = 0 * 64 },
			doc_bag = { x = 2 * 64, y = 7 * 64 },
			body_bag = { x = 5 * 64, y = 11 * 64 },
		}
		function DynamicHUDList.EquipmentItem:init(parent, name, type, equipment, peer_id)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "center", w = parent:panel():h() * 4/5, h = parent:panel():h() })
			
			self._equipment = equipment
			local icon = self.ICON_TABLE[type]
			
			self._box = HUDBGBox_create(self._panel, {
					w = self._panel:w(),
					h = self._panel:h(),
				}, {})
			
			self._icon = self._panel:bitmap({
				name = "icon",
				texture = "guis/textures/pd2/skilltree/icons_atlas",
				texture_rect = { icon.x, icon.y, 64, 64 },
				h = self:panel():w() * 0.8,
				w = self:panel():w() * 0.8,
				blend_mode = "add",
				layer = 0
			})
			self._icon:set_center(self._panel:center())
			self._icon:set_top(self._panel:top())
			
			self._info_text = self._panel:text({
				name = "info",
				text = "",
				align = "center",
				vertical = "bottom",
				w = self._panel:w(),
				h = self._panel:h() * 0.4,
				color = Color.white,
				layer = 1,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._panel:h() * 0.4,
			})
			self._info_text:set_bottom(self._panel:bottom())

			self:set_text_color(peer_id)
		end

		function DynamicHUDList.EquipmentItem:set_text_color(peer_id)
			local color = peer_id and peer_id > 0 and tweak_data.chat_colors[peer_id]:with_alpha(1) or Color(1, 1, 1, 1)
			self._icon:set_color(color)
		end
		
		DynamicHUDList.BagEquipmentItem = DynamicHUDList.BagEquipmentItem or class(DynamicHUDList.EquipmentItem)
		function DynamicHUDList.BagEquipmentItem:init(parent, name, type, equipment, peer_id)
			DynamicHUDList.EquipmentItem.init(self, parent, name, type, equipment, peer_id)
		end
		
		function DynamicHUDList.BagEquipmentItem:set_amount(amount)
			self._current_amount = amount
			self:activate()
			self:_update_info_text()
			if self._current_amount <= 0 then
				self._parent:unregister_item(self._name)
			end
		end
		
		function DynamicHUDList.BagEquipmentItem.set_bag_amount(item, amount)
			local item = managers.hud:get_hud_list("side_list"):item("equipment"):item(item)
			if item then
				item:set_amount(amount)
			end
		end
		
		DynamicHUDList.AmmoBagItem = DynamicHUDList.AmmoBagItem or class(DynamicHUDList.BagEquipmentItem)
		function DynamicHUDList.AmmoBagItem:init(parent, name, equipment, peer_id, upgrade_lvl)
			DynamicHUDList.BagEquipmentItem.init(self, parent, name, "ammo_bag", equipment, peer_id)
			self._max_amount = tweak_data.upgrades.ammo_bag_base + tweak_data.upgrades.values.ammo_bag.ammo_increase[1]
		end
		
		function DynamicHUDList.AmmoBagItem:_update_info_text()
			self._info_text:set_text(string.format("%.0f", self._current_amount * 100) .. "%")
			self._info_text:set_color(self:_get_color_from_table(self._current_amount, self._max_amount))
		end
		
		DynamicHUDList.DocBagItem = DynamicHUDList.DocBagItem or class(DynamicHUDList.BagEquipmentItem)
		function DynamicHUDList.DocBagItem:init(parent, name, equipment, peer_id, upgrade_lvl)
			DynamicHUDList.BagEquipmentItem.init(self, parent, name, "doc_bag", equipment, peer_id)
			self._max_amount = tweak_data.upgrades.doctor_bag_base +  tweak_data.upgrades.values.doctor_bag.amount_increase[1]
		end
		
		function DynamicHUDList.DocBagItem:_update_info_text()
			self._info_text:set_text(string.format("%d", self._current_amount))
			self._info_text:set_color(self:_get_color_from_table(self._current_amount, self._max_amount))
		end

		DynamicHUDList.BodyBagBagItem = DynamicHUDList.BodyBagBagItem or class(DynamicHUDList.BagEquipmentItem)
		function DynamicHUDList.BodyBagBagItem:init(parent, name, equipment, peer_id, upgrade_lvl)
			DynamicHUDList.BagEquipmentItem.init(self, parent, name, "body_bag", equipment, peer_id)
			self._max_amount = tweak_data.upgrades.bodybag_crate_base
		end
		
		function DynamicHUDList.BodyBagBagItem:_update_info_text()
			self._info_text:set_text(string.format("%d", self._current_amount))
			self._info_text:set_color(self:_get_color_from_table(self._current_amount, self._max_amount))
		end

		
		DynamicHUDList.MinionItem = DynamicHUDList.MinionItem or class(DynamicHUDList.ItemBase)
		DynamicHUDList.MinionItem._UNIT_NAMES = {
			security = "Security",
			gensec = "Security",
			cop = "Cop",
			fbi = "FBI",
			swat = "SWAT",
			heavy_swat = "H. SWAT",
			fbi_swat = "FBI SWAT",
			fbi_heavy_swat = "H. FBI SWAT",
			city_swat = "GenSec",
		}
		function DynamicHUDList.MinionItem:init(parent, name, unit, peer_id, upgrade)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "center", w = parent:panel():h() * 4/5, h = parent:panel():h() })
			
			self._unit = unit
			self._max_health = unit:character_damage()._HEALTH_INIT
			unit:character_damage()._is_minion = true
			local type_str = self._UNIT_NAMES[unit:base()._tweak_table] or "UNKNOWN"
		
			self._health_bar = self._panel:bitmap({
				name = "radial_health",
				texture = "guis/textures/pd2/hud_health",
				texture_rect = { 64, 0, -64, 64 },
				render_template = "VertexColorTexturedRadial",
				blend_mode = "add",
				layer = 2,
				color = Color(1, 1, 0, 0),
				w = self._panel:w(),
				h = self._panel:w(),
			})
			self._health_bar:set_bottom(self._panel:bottom())
			
			self._hit_indicator = self._panel:bitmap({
				name = "radial_health",
				texture = "guis/textures/pd2/hud_radial_rim",
				blend_mode = "add",
				layer = 1,
				color = Color.red,
				alpha = 0,
				w = self._panel:w(),
				h = self._panel:w(),
			})
			self._hit_indicator:set_center(self._health_bar:center())

			self._outline = self._panel:bitmap({
				name = "outline",
				texture = "guis/textures/pd2/hud_shield",
				texture_rect = { 64, 0, -64, 64 },
				--render_template = "VertexColorTexturedRadial",
				blend_mode = "add",
				w = self._panel:w() * 0.95,
				h = self._panel:w() * 0.95,
				layer = 1,
				color = Color(1, 1, 1, 1),
			})
			self._outline:set_center(self._health_bar:center())
			
			self._unit_type = self._panel:text({
				name = "type",
				text = type_str,
				align = "center",
				vertical = "top",
				w = self._panel:w(),
				h = self._panel:w() * 0.3,
				color = Color.white,
				layer = 3,
				font = tweak_data.hud_corner.assault_font,
				font_size = math.min(8 / string.len(type_str), 1) * 0.25 * self._panel:h(),
			})

			self:set_health(self._max_health, true)
			self:set_owner(peer_id)
			self:set_upgrade(upgrade)
		end
		
		function DynamicHUDList.MinionItem:set_health(health, skip_animate)
			self._health_bar:set_color(Color(1, health / self._max_health, 1, 1))
			
			if not skip_animate then
				self._hit_indicator:stop()
				self._hit_indicator:animate(callback(self, self, "_animate_damage"))
			end
		end
		
		function DynamicHUDList.MinionItem:set_owner(peer_id)
			self._unit_type:set_color(peer_id and tweak_data.chat_colors[peer_id]:with_alpha(1) or Color(1, 1, 1, 1))
		end
		
		function DynamicHUDList.MinionItem:set_upgrade(upgrade)
			self._outline:set_color(upgrade and Color(1, 0.8, 0.8, 1) or Color(1, 0.3, 0.3, 0.3))
		end
		
		function DynamicHUDList.MinionItem:_animate_damage(icon)
			local duration = 1
			local t = duration
			icon:set_alpha(1)
			
			while t > 0 do
				local dt = coroutine.yield()
				t = math.clamp(t - dt, 0, duration)
				icon:set_alpha(t/duration)
			end
			
			icon:set_alpha(0)
		end
		
		DynamicHUDList.PagerItem = DynamicHUDList.PagerItem or class(DynamicHUDList.ItemBase)
		function DynamicHUDList.PagerItem:init(parent, name, unit)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "left", w = parent:panel():h(), h = parent:panel():h() })
			
			self._unit = unit
			self._max_duration_t = 12
			self._duration_t = self._max_duration_t
			
			self._box = HUDBGBox_create(self._panel, {
					w = self._panel:w(),
					h = self._panel:h(),
				}, {})

			self._timer_text = self._box:text({
				name = "time",
				align = "center",
				vertical = "top",
				w = self._box:w(),
				h = self._box:h(),
				color = Color.red,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._box:h() * 0.5,
				text = string.format("%.1fs", self._duration_t)
			})
			
			self._distance_text = self._box:text({
				name = "distance",
				align = "center",
				vertical = "bottom",
				w = self._box:w(),
				h = self._box:h(),
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = self._box:h() * 0.5,
				text = "DIST"
			})
		end
		
		function DynamicHUDList.PagerItem:set_duration(duration_t)
			self._duration_t = duration_t
		end
		
		function DynamicHUDList.PagerItem:set_answered()
			self._answered = true
			self._timer_text:set_color(Color(1, 0.1, 0.9, 0.1))
		end
		
		function DynamicHUDList.PagerItem:update(t, dt)
			if not self._answered then
				self._duration_t = math.max(self._duration_t - dt, 0)
				self._timer_text:set_text(string.format("%.1fs", self._duration_t))
				self._timer_text:set_color(self:_get_color_from_table(self._duration_t, self._max_duration_t))
				
				if self._duration_t <= 0 then
					self._parent:unregister_item(self._name)
				end
			end

			local distance = 0
			if alive(self._unit) and alive(managers.player:player_unit()) then
				distance = mvector3.normalize(managers.player:player_unit():position() - self._unit:position()) / 100
			end
			self._distance_text:set_text(string.format("%.0fm", distance))
		end	
		
		------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

		DynamicHUDList.BuffItemBase = DynamicHUDList.BuffItemBase or class(DynamicHUDList.ItemBase)
		DynamicHUDList.BuffItemBase.BUFFS = {
			temporary = {
				--Underdog basic
				dmg_multiplier_outnumbered = {
					name = "underdog", 
					type = "temporary",
					atlas = { 2, 1 },
					has_aced = function() return managers.player:has_category_upgrade("temporary", "dmg_dampener_outnumbered") end,
					class = "TimedBuffItem"
				},
				--Underdog aced (redundant, just to make sure this temporary upgrade is handled cleanly)
				dmg_dampener_outnumbered = {
					name = "underdog", 
					type = "temporary",
					atlas = { 2, 1 },
					class = "TimedBuffItem"
				},
				dmg_dampener_outnumbered_strong = {
					name = "overdog", 
					type = "temporary",
					spec = { 6, 4 },
					class = "TimedBuffItem"
				},
				dmg_dampener_close_contact = {	--TODO: Fix level counter? (has 3 levels)
					name = "close_combat", 
					type = "temporary",
					spec = { 5, 4 },
					class = "TimedBuffItem"
				},
				--Combat medic basic
				combat_medic_damage_multiplier = {
					name = "combat_medic", 
					type = "temporary",
					atlas = { 5, 7 },
					class = "TimedBuffItem"
				},
				--Overkill basic/aced
				overkill_damage_multiplier = {
					name = "overkill", 
					type = "temporary",
					atlas = { 3, 2 },
					has_aced = function() return managers.player:has_category_upgrade("player", "overkill_all_weapons") end,
					class = "TimedBuffItem"
				},
				--Bullet storm basic/aced
				no_ammo_cost = {
					name = "bullet_storm", 
					type = "temporary",
					atlas = { 4, 5 },
					class = "TimedBuffItem"
				},
				--Pain killer basic/aced
				passive_revive_damage_reduction = {
					name = "pain_killer", 
					type = "temporary",
					atlas = { 0, 10 },
					has_aced = function(level) return (level and level > 1) end,
					class = "TimedBuffItem"
				},
				--Swan song basic/aced
				berserker_damage_multiplier = {
					name = "swan_song", 
					type = "temporary",
					atlas = { 5, 12 }, 
					has_aced = function() return managers.player:has_category_upgrade("player", "berserker_no_ammo_cost") end,
					ignore = true,
					class = "TimedBuffItem"
				},
				--Quick fix aced
				first_aid_damage_reduction = {
					name = "quick_fix", 
					type = "temporary",
					atlas = { 1, 11 },
					class = "TimedBuffItem"
				},
				--Life drain
				melee_life_leech = {
					name = "life_drain", 
					type = "debuff",
					spec = { 7, 4 },
					icon_color = Color(1, 0.6, 0, 0),
					no_fade = true,
					class = "TimedBuffItem",
				},
			},
			team = {
				stamina = {
					--Endurance aced
					multiplier = {
						name = "endurance", 
						type = "team",
						atlas = { 1, 8 },
						has_aced = function() return true end,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
					--Crew chief tier 3
					passive_multiplier = {
						name = "crew_chief", 
						type = "team",
						atlas = { 2, 7 },
						level = 3,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
					--Crew chief tier 9
					hostage_multiplier = { 
						name = "crew_chief", 
						type = "team",
						atlas = { 2, 7 },
						level = 9,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
				},
				health = {
					--Crew chief tier 5
					passive_multiplier = {
						name = "crew_chief", 
						type = "team",
						atlas = { 2, 7 },
						level = 5,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
					--Crew chief tier 9
					hostage_multiplier = {
						name = "crew_chief", 
						type = "team",
						atlas = { 2, 7 },
						level = 9,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
				},
				armor = {
					--Crew chief tier 7
					multiplier = {
						name = "crew_chief", 
						type = "team",
						atlas = { 2, 7 },
						level = 7,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
					--Bulletproof aced
					regen_time_multiplier = {
						name = "bulletproof", 
						type = "team",
						atlas = { 6, 4 },
						has_aced = function() return true end,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
					--Armorer tier 9
					passive_regen_time_multiplier = {
						name = "armorer", 
						type = "team",
						spec = { 6, 0 },
						level = 9,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
				},
				weapon = {
					--Leadership aced
					recoil_multiplier = {
						name = "leadership", 
						type = "team",
						atlas = { 7, 7 },
						has_aced = function() return true end,
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
				},
				pistol = {
					--Leadership basic
					recoil_multiplier = {
						name = "leadership", 
						type = "team",
						atlas = { 7, 7 },
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
				},
				akimbo = {
					--Leadership basic
					recoil_multiplier = {
						name = "leadership", 
						type = "team",
						atlas = { 7, 7 },
						ace_icon_color = Color(1, 0.2, 1.0, 0.2)
					},
				},
			},
			special = {
				--Inspire basic
				inspire = {
					name = "inspire",
					type = "temporary",
					atlas = { 4, 9 },
					class = "TimedBuffItem"
				},
				--Inspire debuff
				inspire_debuff = {
					name = "inspire_debuff",
					type = "debuff",
					atlas = { 4, 9 },
					icon_color = Color(1, 0.6, 0, 0),
					no_fade = true,
					class = "TimedBuffItem"
				},
				--Berserker basic/aced
				berserker = { 
					name = "berserker", 
					type = "temporary",
					atlas = { 2, 2 }, 
					has_aced = function() return managers.player:has_category_upgrade("player", "damage_health_ratio_multiplier") end,
					class = "BerserkerBuffItem"
				},
				--Bullseye debuff
				bullseye_debuff = {
					name = "bullseye_debuff",
					type = "debuff",
					atlas = { 6, 11 },
					icon_color = Color(1, 0.6, 0, 0),
					no_fade = true,
					class = "TimedBuffItem"
				},
				--Suppression debuff
				suppression = { 
					name = "suppression_debuff", 
					type = "debuff",
					atlas = { 7, 0 }, 
					ignore = true,
					class = "TimedBuffItem",
					no_fade = true,
					icon_color = Color(1, 0.6, 0, 0)
				},
				--Armor regen debuff
				armor_regen = { 
					name = "armor_regen_debuff", 
					type = "debuff",
					spec = { 6, 0 }, 
					class = "TimedBuffItem",
					no_fade = true,
					icon_color = Color(1, 0.6, 0, 0),
					icon_scale = 1.3
				},
				--Trigger happy basic/aced
				trigger_happy = { 
					name = "trigger_happy", 
					type = "temporary",
					atlas = { 7, 11 }, 
					stack = true, 
					stack_limit = tweak_data.upgrades.max_weapon_dmg_mul_stacks or 5,
					class = "TimedBuffItem"
				},
				--Hostage taker basic/aced
				hostage_taker = {
					name = "hostage_taker", 
					type = "temporary",
					atlas = { 2, 10 },
					ignore = true,
					has_aced = function() return managers.player:upgrade_level("player", "hostage_health_regen_addend", 0) > 1 end,
					icon_scale = 1.35
				},
				--Crew chief tier 9 active effect
				hostage_situation = {
					name = "hostage_situation", 
					type = "temporary",
					spec = { 0, 1 }, 
					stack = true, 
					stack_limit = 10
				},
				--Partner in crime basic/aced
				partner_in_crime = {
					name = "partner_in_crime",
					type = "temporary",
					atlas = { 1, 10 },
					has_aced = function() return managers.player:has_category_upgrade("player", "minion_master_health_multiplier") end,
				},
				--Sixth sense basic
				sixth_sense = {
					name = "sixth_sense",
					type = "temporary",
					atlas = { 6, 10 },
					stack = true,
					flash_color = Color.blue,
					class = "TimedBuffItem"
				},
				--Close combat / overdog melee stack damage
				melee_stack_damage = {
					name = "melee_stack_damage",
					type = "temporary",
					spec = { 5, 4 },
					stack = true,
					stack_limit = tweak_data.upgrades.max_melee_weapon_dmg_mul_stacks or 5,
					has_aced = function() return managers.player:upgrade_level("melee", "stacking_hit_damage_multiplier", 0) > 1 end,
					class = "TimedBuffItem"
				},
			},
		}
		function DynamicHUDList.BuffItemBase:init(parent, name, buff, w, h)
			DynamicHUDList.ItemBase.init(self, parent, name, { align = "bottom", w = w or parent:panel():h(), h = h or parent:panel():h() })
					
			local x, y = unpack(buff.atlas or buff.spec or { 0, 0 })
			local texture = buff.atlas and "guis/textures/pd2/skilltree/icons_atlas" or buff.spec and "guis/textures/pd2/specialization/icons_atlas" or buff.texture
			local texture_rect = (buff.atlas or buff.spec) and { x * 64, y * 64, 64, 64 } or buff.rect
			
			self._icon = self._panel:bitmap({
					name = "icon",
					texture = texture,
					texture_rect = texture_rect,
					valign = "center",
					align = "center",
					h = self:panel():w() * 0.7 * (buff.icon_scale or 1),
					w = self:panel():w() * 0.7 * (buff.icon_scale or 1),
					blend_mode = "add",
					layer = 5,
					color = buff.icon_color or Color.white,
			})
			self._icon:set_center(self:panel():center())
			
			self._flash_icon = self._panel:bitmap({
					name = "flash_icon",
					texture = texture,
					texture_rect = texture_rect,
					valign = "center",
					align = "center",
					layer = 10,
					h = self._icon:h(),
					w = self._icon:w(),
					blend_mode = "add",
					color = buff.flash_color or Color.blue,
					alpha = 0,
			})
			self._flash_icon:set_center(self._icon:center())
			
			self._ace_icon = self._panel:bitmap({
				name = "icon_aced",
				texture = "guis/textures/pd2/skilltree/ace",
				texture_rect = { 37, 28, 54, 70 },
				valign = "center",
				align = "center",
				visible = buff.has_aced and buff.has_aced() or false,
				layer = 0,
				alpha = 0.75,
				h = self._icon:h(),
				w = 0.8 * self._icon:w(),
				blend_mode = "add",
				layer = 6,
				color = buff.ace_icon_color or nil,
			})
			self._ace_icon:set_center(self._icon:center())
			
			if buff.level then
				self._level_text = self._panel:text({
					name = "level_text",
					text = "0",
					valign = "center",
					align = "center",
					vertical = "center",
					w = self._icon:w(),
					h = self._icon:h() * 0.5,
					layer = 10,
					color = Color.green,
					blend_mode = "normal",
					font = tweak_data.hud_corner.assault_font,
					font_size = math.round(self._icon:h() * 0.5),
					visible = false
				})
				self._level_text:set_center(self:panel():center())
				self._level_text:set_bottom(self:panel():bottom())
				self._icon:set_top(self:panel():top())
				self._flash_icon:set_center(self._icon:center())
			end
			
			if buff.stack then
				self._stack_limit = buff.stack_limit
				self._stack_text = self._panel:text({
					name = "stack_text",
					text = "0",
					valign = "right",
					align = "right",
					vertical = "right",
					w = self._icon:w(),
					h = self._icon:h() * 0.5,
					layer = 1,
					color = Color.green,
					blend_mode = "normal",
					font = tweak_data.hud_corner.assault_font,
					font_size = math.round(self._icon:h() * 0.5)
				})
				self._stack_text:set_right(self._icon:right())
				self._stack_text:set_bottom(self._icon:bottom())
			end
		end
		
		function DynamicHUDList.BuffItemBase:set_aced(status)
			self._ace_icon:set_visible(status)
		end
		
		function DynamicHUDList.BuffItemBase:set_level(new_level, override)
			if self._level_text then
				self._current_level = override and new_level or math.max(self._current_level or 0, new_level)
				self._level_text:set_text("Tier " .. self._current_level)
				self._level_text:set_visible(self._current_level > 1)
			end
		end
		
		function DynamicHUDList.BuffItemBase:set_stack_count(new_count)
			if new_count <= 0 then
				self:deactivate()
				self._stack_text:set_text("")
			elseif self._stack_text then
				local count = self._stack_limit and math.min(new_count, self._stack_limit) or new_count
				self._stack_text:set_text(tostring(count))
				self:activate()
			end
		end
		
		function DynamicHUDList.BuffItemBase:set_active(status)
			if status then
				self:activate()
			else
				self:deactivate()
			end
		end
		
		function DynamicHUDList.BuffItemBase:flash(duration)
			self._flash_icon:stop()
			self._flash_icon:animate(callback(self, self, "_animate_flash"), duration or 0.5)
		end
		
		function DynamicHUDList.BuffItemBase:_animate_flash(icon, duration)
			self._flash_icon:set_alpha(0)
			self._icon:set_alpha(1)
			
			local t = duration
			while t > 0 do
				local dt = coroutine.yield()
				t = math.max(t - dt, 0)
				local value = math.sin(t/duration * 180)
				self._flash_icon:set_alpha(value)
				self._icon:set_alpha(1-value)
			end

			self._flash_icon:set_alpha(0)
			self._icon:set_alpha(1)
		end

		DynamicHUDList.TimedBuffItem = DynamicHUDList.TimedBuffItem or class(DynamicHUDList.BuffItemBase)
		function DynamicHUDList.TimedBuffItem:init(parent, name, buff)
			DynamicHUDList.BuffItemBase.init(self, parent, name, buff)
			
			self._timer = CircleBitmapGuiObject:new(self._panel, {
				use_bg = true,
				radius = 0.9 * self:panel():w() / 2,
				color = Color(1, 1, 1, 1),
				blend_mode = "add",
				layer = 0
			})
			self._timer._circle:set_center(self._icon:center())
			self:set_current(1)
			
			self._remaining_t = 0
			self._duration_t = 0
		end

		function DynamicHUDList.TimedBuffItem:set_duration(duration)
			self._remaining_t = duration
			self._duration_t = duration
			self:set_current(1)
			self:activate()
		end
		
		function DynamicHUDList.TimedBuffItem:refresh_duration()
			self:set_duration(self._duration_t)
		end

		function DynamicHUDList.TimedBuffItem:update(t, dt)
			self._remaining_t = self._remaining_t - dt
			
			if self._remaining_t <= 0 then
				self:deactivate()
				self:set_current(0)
			else
				self:set_current(self._remaining_t / self._duration_t)
			end
		end
		
		function DynamicHUDList.TimedBuffItem:set_current(ratio)	--TODO: why the hell wont set_current directly on the timer work?
			self._timer._circle:set_color(Color(1, ratio, 1, 1))
		end

		DynamicHUDList.BerserkerBuffItem = DynamicHUDList.BerserkerBuffItem or class(DynamicHUDList.BuffItemBase)
		function DynamicHUDList.BerserkerBuffItem:init(parent, name)
			DynamicHUDList.BuffItemBase.init(self, parent, name, self.BUFFS.special.berserker)
			
			self._text = self._panel:text({
				name = "text",
				text = "0",
				valign = "bottom",
				halign = "center",
				align = "center",
				vertical = "bottom",
				horizontal = "center",
				w = self._icon:w(),
				h = math.round(self._icon:w() * 0.4),
				layer = 0,
				color = Color.white,
				font = tweak_data.hud_corner.assault_font,
				font_size = math.round(self._icon:w() * 0.4),
				blend_mode = "normal"
			})
			self._icon:set_top(self:panel():top() + self._icon:h() * 0.1) --Extra space for ace card
			self._ace_icon:set_center(self._icon:center())
			self._flash_icon:set_center(self._icon:center())
			self._text:set_center(self._icon:center())
			self._text:set_bottom(self:panel():bottom())

			self._has_berserker = managers.player:has_category_upgrade("player", "melee_damage_health_ratio_multiplier") or 
				managers.player:has_category_upgrade("player", "damage_health_ratio_multiplier")
		end

		function DynamicHUDList.BerserkerBuffItem:set_health(ratio)
			local threshold = tweak_data.upgrades.player_damage_health_ratio_threshold
			if self._has_berserker and ratio <= threshold then
				local damage_ratio = 1 - ratio / math.max(0.01, threshold)
				self._text:set_color(self:_get_color_from_table(damage_ratio, 1))
				self._text:set_text(string.format("%.0f", damage_ratio * 100) .. "%")
				self:activate()
			else
				self:deactivate()
			end
		end
		
	end

	if string.lower(RequiredScript) == "lib/units/beings/player/playerdamage" then

		if SHOW_BUFFS then
			local PlayerDamage_set_health_original = PlayerDamage.set_health
			local PlayerDamage_build_suppression_original = PlayerDamage.build_suppression
			local PlayerDamage__damage_screen_original = PlayerDamage._damage_screen
			local PlayerDamage_restore_armor_original = PlayerDamage.restore_armor

			function PlayerDamage:set_health(...)
				PlayerDamage_set_health_original(self, ...)
				managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.berserker, { set_health = self:health_ratio() })
			end
			
			function PlayerDamage:build_suppression(amount, ...)
				if not self:_chk_suppression_too_soon(amount) then
					PlayerDamage_build_suppression_original(self, amount, ...)
				
					if self._supperssion_data.value == tweak_data.player.suppression.max_value then
						managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.suppression, { set_duration = tweak_data.player.suppression.decay_start_delay })
						if self:get_real_armor() < self:_total_armor() then
							managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.armor_regen, { refresh_duration = true })
						end
					end
				end
			end
			
			function PlayerDamage:_damage_screen(...)
				PlayerDamage__damage_screen_original(self, ...)
				local delay = self._regenerate_timer + (self._supperssion_data.decay_start_t and (self._supperssion_data.decay_start_t - managers.player:player_timer():time()) or 0)
				managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.armor_regen, { set_duration = delay })
			end

			function PlayerDamage:restore_armor(...)
				PlayerDamage_restore_armor_original(self, ...)
				managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.bullseye_debuff, { set_duration = tweak_data.upgrades.on_headshot_dealt_cooldown or 0 })
				if self:get_real_armor() >= self:_total_armor() then
					managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.armor_regen, { deactivate = true })
				end
			end
		end
		
	end

	if string.lower(RequiredScript) == "lib/units/beings/player/playermovement" then

		if SHOW_BUFFS then
			local PlayerMovement_on_morale_boost_original = PlayerMovement.on_morale_boost

			function PlayerMovement:on_morale_boost(...)
				PlayerMovement_on_morale_boost_original(self, ...)
				managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.inspire, { set_duration = tweak_data.upgrades.morale_boost_time })
			end
		end

	end

	if string.lower(RequiredScript) == "lib/units/beings/player/states/playerstandard" then

		if SHOW_BUFFS then
			local PlayerStandard__check_action_primary_attack_original = PlayerStandard._check_action_primary_attack
			local PlayerStandard__do_melee_damage_original = PlayerStandard._do_melee_damage
			local PlayerStandard__do_action_intimidate_original = PlayerStandard._do_action_intimidate

			function PlayerStandard:_check_action_primary_attack(t, ...)
				local result = PlayerStandard__check_action_primary_attack_original(self, t, ...)
				if result and self._state_data.stacking_dmg_mul then
					local weapon_category = self._equipped_unit:base():weapon_tweak_data().category
					local stack = self._state_data.stacking_dmg_mul[weapon_category]
					if stack then
						managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.trigger_happy, { set_duration = (stack[1] or 0) - t, set_stack_count = stack[2] })
					end
				end
				return result
			end
			
			function PlayerStandard:_do_melee_damage(t, ...)
				local result = PlayerStandard__do_melee_damage_original(self, t, ...)
				if self._state_data.stacking_dmg_mul then
					local stack = self._state_data.stacking_dmg_mul.melee
					if stack then
						managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.melee_stack_damage, { set_duration = (stack[1] or 0) - t, set_stack_count = stack[2] })
					end
				end
				return result
			end
			
			function PlayerStandard:_do_action_intimidate(t, interact_type, ...)
				if interact_type == "cmd_gogo" or interact_type == "cmd_get_up" then
					managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.inspire_debuff, { set_duration = self._ext_movement:rally_skill_data().morale_boost_cooldown_t or 3.5 })
				end
				return PlayerStandard__do_action_intimidate_original(self, t, interact_type, ...)
			end
			
			function PlayerStandard:_update_omniscience(t, dt)
				if managers.groupai:state():whisper_mode() then
					local action_forbidden = self._unit:base():stats_screen_visible() or not managers.player:has_category_upgrade("player", "standstill_omniscience") or self:_interacting() or self._ext_movement:has_carry_restriction() or self:is_deploying() or self:_changing_weapon() or self:_is_throwing_grenade() or self:_is_meleeing() or self:_on_zipline() or self._moving or self:running() or self:_is_reloading() or self:in_air() or self:in_steelsight() or self:is_equipping() or self:shooting() or not tweak_data.player.omniscience
					if action_forbidden then
						if self._state_data.omniscience_t then
							managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.sixth_sense, { deactivate = true, set_stack_count = 0 })
							self._state_data.omniscience_t = nil
							self._state_data.omniscience_units_detected = {}
						end
						return
					end
					
					if not self._state_data.omniscience_t then
						managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.sixth_sense, { set_duration = tweak_data.player.omniscience.start_t })
						self._state_data.omniscience_t = t + tweak_data.player.omniscience.start_t
					end
					
					if t >= self._state_data.omniscience_t then
						local sensed_targets = World:find_units_quick("sphere", self._unit:movement():m_pos(), tweak_data.player.omniscience.sense_radius * 1.25, managers.slot:get_mask("trip_mine_targets"))
						self._state_data.omniscience_units_detected = self._state_data.omniscience_units_detected or {}
						managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.sixth_sense, { set_stack_count = #sensed_targets })
						for _, unit in ipairs(sensed_targets) do
							if alive(unit) and not tweak_data.character[unit:base()._tweak_table].is_escort and not unit:anim_data().tied and not unit:anim_data().surrender then
								if not self._state_data.omniscience_units_detected[unit:key()] or t >= self._state_data.omniscience_units_detected[unit:key()] then
									self._state_data.omniscience_units_detected[unit:key()] = t + tweak_data.player.omniscience.target_resense_t
									managers.game_play_central:auto_highlight_enemy(unit, true)
									managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.sixth_sense, { flash = tweak_data.player.omniscience.interval_t * 0.9 })
									break
								end
								
							end
						end
						self._state_data.omniscience_t = t + tweak_data.player.omniscience.interval_t
						managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.sixth_sense, { set_duration = tweak_data.player.omniscience.interval_t })
					end
				end
			end
		end
		
	end

	if string.lower(RequiredScript) == "lib/managers/playermanager" then

		if SHOW_BUFFS then
			local PlayerManager_count_up_player_minions_original = PlayerManager.count_up_player_minions
			local PlayerManager_count_down_player_minions_original = PlayerManager.count_down_player_minions
			local PlayerManager_update_hostage_skills_original = PlayerManager.update_hostage_skills
			local PlayerManager_activate_temporary_upgrade_original = PlayerManager.activate_temporary_upgrade
			local PlayerManager_activate_temporary_upgrade_by_level_original = PlayerManager.activate_temporary_upgrade_by_level
			local PlayerManager_deactivate_temporary_upgrade_original = PlayerManager.deactivate_temporary_upgrade
			local PlayerManager_add_synced_team_upgrade_original = PlayerManager.add_synced_team_upgrade
			local PlayerManager_peer_dropped_out_original = PlayerManager.peer_dropped_out

			function PlayerManager:count_up_player_minions(...)
				local result = PlayerManager_count_up_player_minions_original(self, ...)
				if self._local_player_minions > 0 and self:has_category_upgrade("player", "minion_master_speed_multiplier") then
					managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.partner_in_crime, { activate = true })
				end
				return result
			end
			
			function PlayerManager:count_down_player_minions(...)
				local result = PlayerManager_count_down_player_minions_original(self, ...)
				if self._local_player_minions <= 0 then
					managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.partner_in_crime, { deactivate = true })
				end
				return result
			end
			
			function PlayerManager:update_hostage_skills(...)
				local stack_count = (managers.groupai and managers.groupai:state():hostage_count() or 0) + (self:num_local_minions() or 0)
				if self:has_team_category_upgrade("health", "hostage_multiplier") or self:has_team_category_upgrade("stamina", "hostage_multiplier") then
					managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.hostage_situation, { set_stack_count = stack_count })
				end
				
				if self:has_category_upgrade("player", "hostage_health_regen_addend") then
					if stack_count > 0 then
						managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.hostage_taker, { activate = true })
					else
						managers.hud:update_buff_item(DynamicHUDList.BuffItemBase.BUFFS.special.hostage_taker, { deactivate = true })
					end
				end
				
				return PlayerManager_update_hostage_skills_original(self, ...)
			end
			
			function PlayerManager:activate_temporary_upgrade_by_level(category, upgrade, level, ...)
				local upgrade_level = self:upgrade_level(category, upgrade, 0) or 0
				if level > upgrade_level then
					return
				end
				local upgrade_value = self:upgrade_value_by_level(category, upgrade, level, 0)
				if upgrade_value == 0 then
					return
				end

				local buff = DynamicHUDList.BuffItemBase.BUFFS[category] and DynamicHUDList.BuffItemBase.BUFFS[category][upgrade]
				if not buff then
					io.write("Attempting to activate unknown buff: " .. tostring(category) .. ", " .. tostring(upgrade) .. "\n")
				else
					managers.hud:update_buff_item(buff, { 
						set_duration = upgrade_value[2], 
						set_aced = buff.has_aced and buff.has_aced(level) or nil
					})
				end
			
				return PlayerManager_activate_temporary_upgrade_by_level_original(self, category, upgrade, level, ...)
			end
			
			function PlayerManager:activate_temporary_upgrade(category, upgrade, ...)
				local upgrade_value = self:upgrade_value(category, upgrade)
				if upgrade_value ~= 0 then 

					local buff = DynamicHUDList.BuffItemBase.BUFFS[category] and DynamicHUDList.BuffItemBase.BUFFS[category][upgrade]
					if not buff then
						io.write("Attempting to activate unknown buff: " .. tostring(category) .. ", " .. tostring(upgrade) .. "\n")
					else
						managers.hud:update_buff_item(buff, { set_duration = upgrade_value[2] })
					end			
				end
				
				return PlayerManager_activate_temporary_upgrade_original(self, category, upgrade, ...)
			end
			
			function PlayerManager:deactivate_temporary_upgrade(category, upgrade, ...)
				local upgrade_value = self:upgrade_value(category, upgrade)
				if self._temporary_upgrades[category] and upgrade_value ~= 0 then
					local buff = DynamicHUDList.BuffItemBase.BUFFS[category] and DynamicHUDList.BuffItemBase.BUFFS[category][upgrade]
					if not buff then
						io.write("Attempting to deactivate unknown buff: " .. tostring(category) .. ", " .. tostring(upgrade) .. "\n")
					else
						managers.hud:update_buff_item(buff, { deactivate = true })
					end
				end
				
				return PlayerManager_deactivate_temporary_upgrade_original(self, category, upgrade, ...)
			end

			function PlayerManager:add_synced_team_upgrade(peer_id, category, upgrade, ...)
				PlayerManager_add_synced_team_upgrade_original(self, peer_id, category, upgrade, ...)

				local buff = DynamicHUDList.BuffItemBase.BUFFS.team[category] and DynamicHUDList.BuffItemBase.BUFFS.team[category][upgrade]
				if not buff then
					io.write("Attempting to activate unknown team buff: " .. tostring(category) .. ", " .. tostring(upgrade) .. "\n")
				else
					managers.hud:update_buff_item(buff, { activate = true, set_level = buff.level })
					if buff.has_aced and buff.has_aced() then
						managers.hud:update_buff_item(buff, { set_aced = true })
					end
				end
			end
			
			function PlayerManager:peer_dropped_out(peer, ...)
				local peer_id = peer:id()
				local peer_upgrades = {}		
				local had_upgrades = false
				for category, data in pairs(managers.player._global.synced_team_upgrades[peer_id] or {}) do
					for upgrade, value in pairs(data or {}) do
						had_upgrades = true
						local buff = DynamicHUDList.BuffItemBase.BUFFS.team[category] and DynamicHUDList.BuffItemBase.BUFFS.team[category][upgrade]
						if not buff then
							io.write("Attempting to activate reset team buff: " .. tostring(category) .. ", " .. tostring(upgrade) .. "\n")
						else
							managers.hud:update_buff_item(buff, { deactivate = true, set_level = 0, set_aced = false })
						end
					end
				end
				
				PlayerManager_peer_dropped_out_original(self, peer, ...)
				
				if not had_upgrades then return end
				
				for category, upgrade_data in pairs(DynamicHUDList.BuffItemBase.BUFFS.team) do
					for upgrade, buff in pairs(upgrade_data) do
						if managers.player:has_team_category_upgrade(category, upgrade) then
							managers.hud:update_buff_item(buff, { activate = true, set_level = buff.level })
							if buff.has_aced and buff.has_aced() then
								managers.hud:update_buff_item(buff, { set_aced = true})
							end
						end
					end
				end
			end
		end

	end

	if string.lower(RequiredScript) == "lib/units/props/timergui" then

		if SHOW_TIMERS then
			local TimerGui__start_original = TimerGui._start
			local TimerGui__set_done_original = TimerGui._set_done
			local TimerGui__set_jammed_original = TimerGui._set_jammed
			local TimerGui__set_powered = TimerGui._set_powered
			local TimerGui_destroy_original = TimerGui.destroy
			local TimerGui_update_original = TimerGui.update
			
			function TimerGui:_start(...)
				if self:_create_list_item() then
					self._list_item:activate()
				end
					
				return TimerGui__start_original(self, ...)
			end
			
			function TimerGui:_set_done(...)
				self:_delete_list_item()
				
				return TimerGui__set_done_original(self, ...)
			end
			
			function TimerGui:_set_jammed(jammed, ...)
				if self._list_item then
					self._list_item:set_jammed(jammed)
				end
				
				return TimerGui__set_jammed_original(self, jammed, ...)
			end
			
			function TimerGui:_set_powered(powered, ...)
				if self._list_item then
					self._list_item:set_powered(powered)
				end
				
				return TimerGui__set_powered(self, powered, ...)
			end
			
			function TimerGui:destroy(...)
				self:_delete_list_item()
				
				return TimerGui_destroy_original(self, ...)
			end
			
			function TimerGui:update(unit, t, ...)
				if self._list_item then
					self._list_item:update_timer(t, self._time_left)
				end
				
				return TimerGui_update_original(self, unit, t, ...)
			end
			
			function TimerGui:_create_list_item()
				if not self._list_item then
					local id = "timer_" .. tostring(self._unit:key())
					local timer_type = self._unit:base().is_drill and "DrillItem" or 
						self._unit:base().is_hacking_device and "HackItem" or
						self._unit:base().is_saw and "SawItem"
					self._list_item =	timer_type and managers.hud:get_hud_list("side_list"):item("timers"):register_item(id, timer_type, self._unit) or
						managers.hud:get_hud_list("side_list"):item("timers"):register_item(id, "TimerItem", self._unit, "?")
				end
				return self._list_item
			end
			
			function TimerGui:_delete_list_item()
				if self._list_item then
					self._list_item:delete()
					self._list_item = nil
				end
			end
		end

	end
		
	if string.lower(RequiredScript) == "lib/units/props/digitalgui" then

		if SHOW_TIMERS then
			local DigitalGui_update_original = DigitalGui.update
			local DigitalGui_timer_start_count_up_original = DigitalGui.timer_start_count_up
			local DigitalGui_timer_start_count_down_original = DigitalGui.timer_start_count_down
			local DigitalGui_timer_pause_original = DigitalGui.timer_pause
			local DigitalGui_timer_resume_original = DigitalGui.timer_resume
			local DigitalGui__timer_stop_original = DigitalGui._timer_stop
			local DigitalGui_destroy_original = DigitalGui.destroy
			local DigitalGui_load_original = DigitalGui.load

			function DigitalGui:update(unit, t, ...)
				DigitalGui_update_original(self, unit, t, ...)
				
				if self._list_item then
					self._list_item:activate()
					self._list_item:update_timer(t, self._timer)
				end
			end
			
			function DigitalGui:timer_start_count_up(...)
				if self:_create_list_item() then
					self._list_item:activate()
					self._list_item:set_jammed(false)
				end
				
				return DigitalGui_timer_start_count_up_original(self, ...)
			end

			function DigitalGui:timer_start_count_down(...)
				if self:_create_list_item() then
					self._list_item:activate()
					self._list_item:set_jammed(false)
				end
				
				return DigitalGui_timer_start_count_down_original(self, ...)
			end

			function DigitalGui:timer_pause(...)
				if self._list_item then
					self._list_item:set_jammed(true)
				end
				
				return DigitalGui_timer_pause_original(self, ...)
			end
			
			function DigitalGui:timer_resume(...)
				if self._list_item then
					self._list_item:set_jammed(false)
				end
				
				return DigitalGui_timer_resume_original(self, ...)
			end

			function DigitalGui:_timer_stop(...)
				self:_delete_list_item()
				
				return DigitalGui__timer_stop_original(self, ...)
			end

			function DigitalGui:destroy(...)
				self:_delete_list_item()
				
				return DigitalGui_destroy_original(self, ...)
			end

			function DigitalGui:load(...)
				DigitalGui_load_original(self, ...)
				
				self:_create_list_item()
			end
			
			function DigitalGui:_create_list_item()
				self._list_item = self._list_item or self:is_timer() and managers.hud:get_hud_list("side_list"):item("timers"):register_item("timer_" .. tostring(self._unit:key()), "DigitalTimerItem", self._unit)
				return self._list_item
			end
			
			function DigitalGui:_delete_list_item()
				if self._list_item then
					self._list_item:delete()
					self._list_item = nil
				end
			end
		end
		
	end

	if string.lower(RequiredScript) == "lib/units/interactions/interactionext" then
		
		if SHOW_TIMERS then
			local MissionDoorDeviceInteractionExt_check_for_upgrade_original = MissionDoorDeviceInteractionExt.check_for_upgrade
		
			function MissionDoorDeviceInteractionExt:check_for_upgrade(...)
				local u_base = self._unit:base()
				local timer_gui = self._unit:timer_gui()
				if timer_gui and timer_gui:is_visible() and u_base and timer_gui._upgrade_tweak_data and u_base.get_skill_upgrades then
					local unit_info_table = u_base:get_skill_upgrades()
					local can_upgrade = false
					for i in pairs(self:split_info_id(self:get_player_info_id())) do
						if not unit_info_table[i] then
							can_upgrade = true
							break
						end
					end
					managers.hud:get_hud_list("side_list"):item("timers"):item("timer_" .. tostring(self._unit:key())):set_can_upgrade(can_upgrade)
				end
				
				return MissionDoorDeviceInteractionExt_check_for_upgrade_original(self, ...)
			end
		end
		
		if SHOW_PAGERS then
			local IntimitateInteractionExt_sync_interacted_original = IntimitateInteractionExt.sync_interacted
			local IntimitateInteractionExt_interact_original = IntimitateInteractionExt.interact
			local IntimitateInteractionExt__at_interact_interupt_original = IntimitateInteractionExt._at_interact_interupt
			local IntimitateInteractionExt__at_interact_start_original = IntimitateInteractionExt._at_interact_start
			
			function IntimitateInteractionExt:sync_interacted(peer, player, status, ...)
				if self.tweak_data == "corpse_alarm_pager" and (status == 2 or status == 3) then
					managers.hud:get_hud_list("side_list"):item("pagers"):unregister_item(tostring(self._unit:key()))
				end
				
				return IntimitateInteractionExt_sync_interacted_original(self, peer, player, status, ...)
			end

			function IntimitateInteractionExt:interact(...)
				if self.tweak_data == "corpse_alarm_pager" then
					managers.hud:get_hud_list("side_list"):item("pagers"):unregister_item(tostring(self._unit:key()))
				end

				return IntimitateInteractionExt_interact_original(self, ...)
			end
			
			function IntimitateInteractionExt:_at_interact_start(...)
				if self.tweak_data == "corpse_alarm_pager" then
					local item = managers.hud:get_hud_list("side_list"):item("pagers"):register_item(tostring(self._unit:key()), "PagerItem", self._unit)
					item:activate()
					item:set_answered()
				end
			
				return IntimitateInteractionExt__at_interact_start_original(self, ...)
			end
			
			function IntimitateInteractionExt:_at_interact_interupt(player, complete, ...)
				if self.tweak_data == "corpse_alarm_pager" and not complete then
					managers.hud:get_hud_list("side_list"):item("pagers"):unregister_item(tostring(self._unit:key()))
				end
			
				return IntimitateInteractionExt__at_interact_interupt_original(self, player, complete, ...)
			end

			function IntimitateInteractionExt:destroy(...)
				if self.tweak_data == "corpse_alarm_pager" then
					managers.hud:get_hud_list("side_list"):item("pagers"):unregister_item(tostring(self._unit:key()))
				end
				
				return IntimitateInteractionExt.super.destroy(self, ...)
			end	
			
			function IntimitateInteractionExt:set_active(active, ...)
				if Network:is_server() and self.tweak_data == "corpse_alarm_pager" then
					if active then
						managers.hud:get_hud_list("side_list"):item("pagers"):register_item(tostring(self._unit:key()), "PagerItem", self._unit):activate()
					else
						managers.hud:get_hud_list("side_list"):item("pagers"):unregister_item(tostring(self._unit:key()))
					end
				end
				
				return IntimitateInteractionExt.super.set_active(self, active, ...)
			end	
		end
		
	end

	if string.lower(RequiredScript) == "lib/units/equipment/doctor_bag/doctorbagbase" then

		if SHOW_EQUIPMENT then
			local DoctorBagBase_spawn_original = DoctorBagBase.spawn
			local DoctorBagBase_sync_setup_original = DoctorBagBase.sync_setup
			local DoctorBagBase_sync_taken_original = DoctorBagBase.sync_taken
			local DoctorBagBase__take_original = DoctorBagBase._take
			local DoctorBagBase_load_original = DoctorBagBase.load
			local DoctorBagBase_destroy_original = DoctorBagBase.destroy

			function DoctorBagBase.spawn(pos, rot, amount_upgrade_lvl, peer_id)
				local unit = DoctorBagBase_spawn_original(pos, rot, amount_upgrade_lvl, peer_id)
				if unit then
					managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(unit:key()), "DocBagItem", unit:base(), peer_id, amount_upgrade_lvl):set_amount(unit:base()._amount)
				end
				return unit
			end

			function DoctorBagBase:sync_setup(amount_upgrade_lvl, peer_id)
				DoctorBagBase_sync_setup_original(self, amount_upgrade_lvl, peer_id)
				managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(self._unit:key()), "DocBagItem", self, peer_id, amount_upgrade_lvl):set_amount(self._amount)
			end
			
			function DoctorBagBase:sync_taken(...)
				DoctorBagBase_sync_taken_original(self, ...)
				DynamicHUDList.BagEquipmentItem.set_bag_amount(tostring(self._unit:key()), self._amount)
			end
			
			function DoctorBagBase:_take(...)
				local result = DoctorBagBase__take_original(self, ...)
				DynamicHUDList.BagEquipmentItem.set_bag_amount(tostring(self._unit:key()), self._amount)
				return result
			end
			
			function DoctorBagBase:load(...)
				DoctorBagBase_load_original(self, ...)
				managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(self._unit:key()), "DocBagItem", self):set_amount(self._amount)
			end
			
			function DoctorBagBase:destroy(...)
				DoctorBagBase_destroy_original(self, ...)
				managers.hud:get_hud_list("side_list"):item("equipment"):unregister_item(tostring(self._unit:key()))
			end
		end
		
	end

	if string.lower(RequiredScript) == "lib/units/equipment/ammo_bag/ammobagbase" then

		if SHOW_EQUIPMENT then
			local AmmoBagBase_spawn_original = AmmoBagBase.spawn
			local AmmoBagBase_sync_setup_original = AmmoBagBase.sync_setup
			local AmmoBagBase_sync_ammo_taken_original = AmmoBagBase.sync_ammo_taken
			local AmmoBagBase__take_ammo_original = AmmoBagBase._take_ammo
			local AmmoBagBase_load_original = AmmoBagBase.load
			local AmmoBagBase_destroy_original = AmmoBagBase.destroy
			
			function AmmoBagBase.spawn(pos, rot, ammo_upgrade_lvl, peer_id)
				local unit = AmmoBagBase_spawn_original(pos, rot, ammo_upgrade_lvl, peer_id)
				if unit then
					managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(unit:key()), "AmmoBagItem", unit:base(), peer_id, ammo_upgrade_lvl):set_amount(unit:base()._ammo_amount)
				end
				return unit
			end

			function AmmoBagBase:sync_setup(ammo_upgrade_lvl, peer_id)
				AmmoBagBase_sync_setup_original(self, ammo_upgrade_lvl, peer_id)
				managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(self._unit:key()), "AmmoBagItem", self, peer_id, ammo_upgrade_lvl):set_amount(self._ammo_amount)
			end
			
			function AmmoBagBase:sync_ammo_taken(...)
				AmmoBagBase_sync_ammo_taken_original(self, ...)
				DynamicHUDList.BagEquipmentItem.set_bag_amount(tostring(self._unit:key()), self._ammo_amount)
			end
			
			function AmmoBagBase:_take_ammo(...)
				local result = AmmoBagBase__take_ammo_original(self, ...)
				DynamicHUDList.BagEquipmentItem.set_bag_amount(tostring(self._unit:key()), self._ammo_amount)
				return result
			end
			
			function AmmoBagBase:load(...)
				AmmoBagBase_load_original(self, ...)
				managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(self._unit:key()), "AmmoBagItem", self):set_amount(self._ammo_amount)
			end
			
			function AmmoBagBase:destroy(...)
				AmmoBagBase_destroy_original(self, ...)
				managers.hud:get_hud_list("side_list"):item("equipment"):unregister_item(tostring(self._unit:key()))
			end
		end

	end

	if string.lower(RequiredScript) == "lib/units/equipment/bodybags_bag/bodybagsbagbase" then

		if SHOW_EQUIPMENT then
			local BodyBagsBagBase_spawn_original = BodyBagsBagBase.spawn
			local BodyBagsBagBase_sync_setup_original = BodyBagsBagBase.sync_setup
			local BodyBagsBagBase_sync_bodybag_taken_original = BodyBagsBagBase.sync_bodybag_taken
			local BodyBagsBagBase_take_bodybag_original = BodyBagsBagBase.take_bodybag
			local BodyBagsBagBase_load_original = BodyBagsBagBase.load
			local BodyBagsBagBase_destroy_original = BodyBagsBagBase.destroy
			
			function BodyBagsBagBase.spawn(pos, rot, upgrade_lvl, peer_id)
				local unit = BodyBagsBagBase_spawn_original(pos, rot, upgrade_lvl, peer_id)
				if unit then
					managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(unit:key()), "BodyBagBagItem", unit:base(), peer_id):set_amount(unit:base()._bodybag_amount)
				end
				return unit
			end

			function BodyBagsBagBase:sync_setup(upgrade_lvl, peer_id)
				BodyBagsBagBase_sync_setup_original(self, upgrade_lvl, peer_id)
				managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(self._unit:key()), "BodyBagBagItem", self, peer_id, upgrade_lvl):set_amount(self._bodybag_amount)
			end
			
			function BodyBagsBagBase:sync_bodybag_taken(...)
				BodyBagsBagBase_sync_bodybag_taken_original(self, ...)
				DynamicHUDList.BagEquipmentItem.set_bag_amount(tostring(self._unit:key()), self._bodybag_amount)
			end
			
			function BodyBagsBagBase:take_bodybag(...)
				local result = BodyBagsBagBase_take_bodybag_original(self, ...)
				DynamicHUDList.BagEquipmentItem.set_bag_amount(tostring(self._unit:key()), self._bodybag_amount)
				return result
			end
			
			function BodyBagsBagBase:load(...)
				BodyBagsBagBase_load_original(self, ...)
				managers.hud:get_hud_list("side_list"):item("equipment"):register_item(tostring(self._unit:key()), "BodyBagBagItem", self):set_amount(self._bodybag_amount)
			end
			
			function BodyBagsBagBase:destroy(...)
				BodyBagsBagBase_destroy_original(self, ...)
				managers.hud:get_hud_list("side_list"):item("equipment"):unregister_item(tostring(self._unit:key()))
			end
		end

	end

	if string.lower(RequiredScript) == "lib/units/equipment/ecm_jammer/ecmjammerbase" then

		if SHOW_ECMS then
			local ECMJammerBase_setup_original = ECMJammerBase.setup
			local ECMJammerBase_sync_setup_original = ECMJammerBase.sync_setup
			local ECMJammerBase_destroy_original = ECMJammerBase.destroy
			local ECMJammerBase_load_original = ECMJammerBase.load
			local ECMJammerBase_update_original = ECMJammerBase.update

			function ECMJammerBase:sync_setup(upgrade_lvl, ...)
				ECMJammerBase_sync_setup_original(self, upgrade_lvl, ...)
				
				managers.hud:get_hud_list("right_side_list"):item("ecm_list"):register_item("ECM_" .. tostring(self._unit:key()), "ECMItem"):activate()
			end
			
			function ECMJammerBase:setup(battery_life_upgrade_lvl, ...)
				ECMJammerBase_setup_original(self, battery_life_upgrade_lvl, ...)
				
				managers.hud:get_hud_list("right_side_list"):item("ecm_list"):register_item("ECM_" .. tostring(self._unit:key()), "ECMItem"):activate()
			end
			
			function ECMJammerBase:destroy(...)
				local item = managers.hud:get_hud_list("right_side_list"):item("ecm_list"):unregister_item("ECM_" .. tostring(self._unit:key()))
				
				return ECMJammerBase_destroy_original(self, ...)
			end
			
			function ECMJammerBase:load(...)
				ECMJammerBase_load_original(self, ...)
				
				managers.hud:get_hud_list("right_side_list"):item("ecm_list"):register_item("ECM_" .. tostring(self._unit:key()), "ECMItem"):activate()
			end
			
			function ECMJammerBase:update(unit, t, ...)
				ECMJammerBase_update_original(self, unit, t, ...)

				local item = managers.hud:get_hud_list("right_side_list"):item("ecm_list"):item("ECM_" .. tostring(self._unit:key()))
				if item then
					item:update_timer(t, self._battery_life)
				end
			end
		end
		
	end

	if string.lower(RequiredScript) == "lib/managers/group_ai_states/groupaistatebase" then

		if SHOW_MINIONS then
			local GroupAIStateBase_convert_hostage_to_criminal_original = GroupAIStateBase.convert_hostage_to_criminal
			local GroupAIStateBase_sync_converted_enemy_original = GroupAIStateBase.sync_converted_enemy

			function GroupAIStateBase:convert_hostage_to_criminal(unit, peer_unit, ...)
				GroupAIStateBase_convert_hostage_to_criminal_original(self, unit, peer_unit, ...)
				
				if unit:brain()._logic_data.is_converted then
					local peer_id = peer_unit and managers.network:game():member_from_unit(peer_unit):peer():id() or managers.network:session():local_peer():id()
					local owner_base = peer_unit and peer_unit:base() or managers.player
					local upgrade = (owner_base:upgrade_level("player", "convert_enemies_health_multiplier") or 0) > 0
					managers.hud:get_hud_list("side_list"):item("minions"):register_item(tostring(unit:key()), "MinionItem", unit, peer_id, upgrade):activate()
				end
			end
			
			function GroupAIStateBase:sync_converted_enemy(converted_enemy, ...)
				GroupAIStateBase_sync_converted_enemy_original(self, converted_enemy, ...)
				managers.hud:get_hud_list("side_list"):item("minions"):register_item(tostring(converted_enemy:key()), "MinionItem", converted_enemy):activate()
			end
		end
		
		if SHOW_CIVILIANS or SHOW_HOSTAGES then
			local GroupAIStateBase_update_original = GroupAIStateBase.update
			local GroupAIStateBase_on_hostage_state_original = GroupAIStateBase.on_hostage_state
			local GroupAIStateBase_sync_hostage_headcount_original = GroupAIStateBase.sync_hostage_headcount
			
			function GroupAIStateBase:update(t, ...)
				if self._client_hostage_count_expire_t and t < self._client_hostage_count_expire_t then
					self:_client_hostage_count_cbk()
				end
				
				return GroupAIStateBase_update_original(self, t, ...)
			end	
			
			function GroupAIStateBase:on_hostage_state(...)
				GroupAIStateBase_on_hostage_state_original(self, ...)
				self:_update_hostage_count()
			end
			
			function GroupAIStateBase:sync_hostage_headcount(...)
				GroupAIStateBase_sync_hostage_headcount_original(self, ...)
				
				if Network:is_server() then
					self:_update_hostage_count()
				else
					self._client_hostage_count_expire_t = self._t + 10
				end
			end
			
			function GroupAIStateBase:_client_hostage_count_cbk()
				local police_count = 0
				for k, v in pairs(managers.enemy:all_enemies()) do
					if v and v.unit and v.unit.anim_data and v.unit:anim_data() then
						if v.unit:anim_data().surrender then
							police_count = police_count + 1
						end
					end
				end
				
				self._civilian_hostages = self:hostage_count() - police_count
				self._police_hostage_headcount = police_count
				self:_update_hostage_count()
			end
			
			function GroupAIStateBase:civilian_hostage_count()
				return self._civilian_hostages or 0
			end
			
			function GroupAIStateBase:_update_hostage_count()
				if Network:is_server() then
					self._civilian_hostages = (self._hostage_headcount or 0) - self._police_hostage_headcount
					if SHOW_HOSTAGES then
						managers.hud:get_hud_list("right_side_list"):item("hostage_count_list"):item("CopHostages"):set_free_slots(self:has_room_for_police_hostage()) 
					end
				end
				
				if SHOW_HOSTAGES then
					managers.hud:get_hud_list("right_side_list"):item("hostage_count_list"):item("CivHostages"):set_count(self:civilian_hostage_count())
					managers.hud:get_hud_list("right_side_list"):item("hostage_count_list"):item("CopHostages"):set_count(self:police_hostage_count())
				end
				
				if SHOW_CIVILIANS then
					managers.hud:get_hud_list("right_side_list"):item("hostage_count_list"):item("CivCount"):set_count(managers.enemy:get_unit_count("civilian") - self:civilian_hostage_count())
				end
			end
		end

	end
		
	if string.lower(RequiredScript) == "lib/units/enemies/cop/copdamage" then
		
		if SHOW_MINIONS then
			local CopDamage__on_damage_received_original = CopDamage._on_damage_received
			
			function CopDamage:_on_damage_received(damage_info, ...)
				if self._is_minion then
					managers.hud:get_hud_list("side_list"):item("minions"):item(tostring(self._unit:key())):set_health(self._health)
				end
			
				return CopDamage__on_damage_received_original(self, damage_info, ...)
			end
		end

	end
		
	if string.lower(RequiredScript) == "lib/network/handlers/unitnetworkhandler" then

		if SHOW_MINIONS then
			local UnitNetworkHandler_mark_minion_original = UnitNetworkHandler.mark_minion
			local UnitNetworkHandler_hostage_trade_original = UnitNetworkHandler.hostage_trade
			local UnitNetworkHandler_unit_traded_original = UnitNetworkHandler.unit_traded
		
			function UnitNetworkHandler:mark_minion(unit, minion_owner_peer_id, convert_enemies_health_multiplier_level, passive_convert_enemies_health_multiplier_level, ...)
				UnitNetworkHandler_mark_minion_original(self, unit, minion_owner_peer_id, convert_enemies_health_multiplier_level, passive_convert_enemies_health_multiplier_level, ...)
				
				if alive(unit) and not unit:character_damage():dead() then
					local has_upgrade = convert_enemies_health_multiplier_level and convert_enemies_health_multiplier_level > 0
					local item = managers.hud:get_hud_list("side_list"):item("minions"):register_item(tostring(unit:key()), "MinionItem", unit, minion_owner_peer_id, has_upgrade)
					item:set_owner(minion_owner_peer_id)
					item:set_upgrade(has_upgrade)
					item:activate()
				end
			end	
			
			function UnitNetworkHandler:hostage_trade(unit, enable, trade_success, ...)
				if not self._verify_gamestate(self._gamestate_filter.any_ingame) or not self._verify_character(unit) then
					return
				end
				
				managers.hud:get_hud_list("side_list"):item("minions"):unregister_item(tostring(unit:key()))
				return UnitNetworkHandler_hostage_trade_original(self, unit, enable, trade_success, ...)
			end
			
			function UnitNetworkHandler:unit_traded(unit, trader, ...)
				if not self._verify_gamestate(self._gamestate_filter.any_ingame) or not self._verify_character(unit) then
					return
				end
				
				managers.hud:get_hud_list("side_list"):item("minions"):unregister_item(tostring(unit:key()))
				return UnitNetworkHandler_unit_traded_original(self, unit, trader, ...)
			end
		end
		
		if SHOW_PAGERS then
			local UnitNetworkHandler_interaction_set_active_original = UnitNetworkHandler.interaction_set_active
			local UnitNetworkHandler_alarm_pager_interaction_original = UnitNetworkHandler.alarm_pager_interaction
		
			function UnitNetworkHandler:interaction_set_active(unit, u_id, active, tweak_data, flash, sender, ...)
				if self._verify_gamestate(self._gamestate_filter.any_ingame) and self._verify_sender(sender) then
					if tweak_data == "corpse_alarm_pager" then
						if not alive(unit) then
							local u_data = managers.enemy:get_corpse_unit_data_from_id(u_id)
							if not u_data then return end
							unit = u_data.unit
						end
						
						local key = tostring(unit:key())
						local item = managers.hud:get_hud_list("side_list"):item("pagers"):item(key)
						
						if item then
							if active then
								if not flash then --Existing pager, non-flash update -> answered pager
									item:set_answered()
								else	--Flash update, unanswered pager -> update duration just because..
									item:set_duration(6)
								end
							else --Inactive pager -> remove item
								item:delete()
							end
						else	--New pager, add item
							managers.hud:get_hud_list("side_list"):item("pagers"):register_item(key, "PagerItem", unit):activate()
						end
					end
				end

				return UnitNetworkHandler_interaction_set_active_original(self, unit, u_id, active, tweak_data, flash, sender, ...)
			end

			function UnitNetworkHandler:alarm_pager_interaction(u_id, tweak_table, status, sender, ...)
				if self._verify_gamestate(self._gamestate_filter.any_ingame) then
					local unit_data = managers.enemy:get_corpse_unit_data_from_id(u_id)
					if unit_data and unit_data.unit:interaction():active() and unit_data.unit:interaction().tweak_data == tweak_table then
						local peer = self._verify_sender(sender)
						if peer then
							local key = tostring(unit_data.unit:key())
							if status == 1 then --Answered pager -> set answered
								local item = managers.hud:get_hud_list("side_list"):item("pagers"):register_item(key, "PagerItem", unit_data.unit)
								item:activate()
								item:set_answered()
							else --Complete or interrupted pager -> remove
								managers.hud:get_hud_list("side_list"):item("pagers"):unregister_item(key)
							end
						end
					end
				end
			
				return UnitNetworkHandler_alarm_pager_interaction_original(self, u_id, tweak_table, status, sender, ...)
			end
		end

	end
		
	if string.lower(RequiredScript) == "lib/managers/enemymanager" then
		
		if SHOW_ENEMIES or SHOW_CIVILIANS then
			local EnemyManager_init_original = EnemyManager.init
		
			EnemyManager._UNIT_TYPE_EXCEPTIONS = {
				drunk_pilot = true,
				escort = true,
				old_hoxton_mission = true,
			}
			
			EnemyManager._UNIT_TYPES = {
				tank = "tank",
				spooc = "spooc",
				taser = "taser",
				shield = "shield",
				sniper = "sniper",
				mobster_boss = "mobster_boss",
				gangster = "thug",
				mobster = "thug",
				biker_escape = "thug",
				security = "security",
				gensec = "security",
				civilian = "civilian",
				civilian_female = "civilian",
				bank_manager = "civilian",
			}
			
			function EnemyManager:init(...)
				EnemyManager_init_original(self, ...)
				
				self._current_unit_count = {}
				for tweak, utype in pairs(self._UNIT_TYPES) do
					self._current_unit_count[utype] = 0
				end
				self._current_unit_count.cop = 0
			end
			
			function EnemyManager:_get_unit_type(unit)
				local tweak = unit:base()._tweak_table
				if not self._UNIT_TYPE_EXCEPTIONS[tweak] then
					local utype = self._UNIT_TYPES[tweak] or "cop"
					if AGGREGATE_ENEMIES and utype ~= "civilian" then
						utype = "cop"
					end
					
					return utype
				end
			end
		
			function EnemyManager:_on_unit_count_change(unit, change)
				local utype = self:_get_unit_type(unit)
				if utype then
					self._current_unit_count[utype] = self._current_unit_count[utype] + change
					if managers.hud then
						managers.hud:update_unit_count(utype, self._current_unit_count[utype])
					end
				end
			end
			
			function EnemyManager:get_unit_count(utype)
				return self._current_unit_count[AGGREGATE_ENEMIES and "cop" or utype]
			end
		end
		
		if SHOW_ENEMIES then
			local register_enemy_original = EnemyManager.register_enemy
		
			function EnemyManager:register_enemy(enemy, ...)
				self:_on_unit_count_change(enemy, 1)
				
				return register_enemy_original(self, enemy, ...)
			end
		end
			
		if SHOW_ENEMIES or SHOW_MINIONS then
			local on_enemy_died_original = EnemyManager.on_enemy_died
			local on_enemy_destroyed_original = EnemyManager.on_enemy_destroyed
			
			local function check_minion(unit)
				if unit:character_damage() and unit:character_damage()._is_minion then
					local item = managers.hud:get_hud_list("side_list"):item("minions"):unregister_item(tostring(unit:key()))
				end
			end
			
			function EnemyManager:on_enemy_died(dead_unit, ...)
				if SHOW_ENEMIES then
					self:_on_unit_count_change(dead_unit, -1)
				end
				
				if SHOW_MINIONS then
					check_minion(dead_unit)
				end
				
				return on_enemy_died_original(self, dead_unit, ...)
			end

			function EnemyManager:on_enemy_destroyed(enemy, ...)
				if SHOW_ENEMIES then
					if enemy and enemy:character_damage() and not enemy:character_damage():dead() then
						self:_on_unit_count_change(enemy, -1)
					end
				end
				
				if SHOW_MINIONS then
					check_minion(enemy)
				end
				
				return on_enemy_destroyed_original(self, enemy, ...)
			end
		end
		
		if SHOW_CIVILIANS then
			local register_civilian_original = EnemyManager.register_civilian
			local on_civilian_died_original = EnemyManager.on_civilian_died
			local on_civilian_destroyed_original = EnemyManager.on_civilian_destroyed
		
			function EnemyManager:register_civilian(unit, ...)
				self:_on_unit_count_change(unit, 1)
				
				return register_civilian_original(self, unit, ...)
			end
			
			function EnemyManager:on_civilian_died(dead_unit, ...)
				self:_on_unit_count_change(dead_unit, -1)
				
				return on_civilian_died_original(self, dead_unit, ...)
			end
			
			function EnemyManager:on_civilian_destroyed(enemy, ...)
				if enemy and enemy:character_damage() and not enemy:character_damage():dead() then
					self:_on_unit_count_change(enemy, -1)
				end
				
				return on_civilian_destroyed_original(self, enemy, ...)
			end
		end
		
	end

	if string.lower(RequiredScript) == "lib/managers/objectinteractionmanager" then

		if SHOW_LOOT then
			ObjectInteractionManager.LOOT_TYPES = {
				gold =			"Gold",
				money =		"Money",
				jewelry =		"Jewelry",
				painting =		"Painting",
				coke =			"Coke",
				meth =			"Meth",
				weapon =		"Weapon",
				server =		"Server",
				turret =			"Turret",
				shell =			"Shell",
				artifact =		"Artifact",
				armor =			"Armor",
				toast =			"Toast",
				diamond =		"Diamond",
				bomb =			"Bomb",
				container =	"?",
			}
			
			ObjectInteractionManager.INTERACTION_ID_TO_TYPE = {
				weapon_case =			"weapon",
				samurai_armor =		"armor",
				--Crates and suitcases etc interaction ID's here -> type "container"
			}
			
			ObjectInteractionManager.CARRY_ID_TO_TYPE = {
				gold =						"gold",
				money =					"money",
				diamonds =				"jewelry",
				painting =					"painting",
				mus_artifact_paint =	"painting",
				coke =						"coke",
				coke_pure =				"coke",
				meth =						"meth",
				weapon =					"weapon",
				circuit =					"server",
				turret =						"turret",
				ammo =					"shell",
				artifact_statue =		"artifact",
				mus_artifact =			"artifact",
				samurai_suit =			"armor",
				sandwich =				"toast",
				hope_diamond =		"diamond",
				cro_loot1 =				"bomb",
				cro_loot2 =				"bomb",
			}
			
			ObjectInteractionManager.LEVEL_COMPENSATION = {
				watchdogs_2 = 			{ coke = 10, },
				watchdogs_2_day =	{ coke = 10, },
				family = 					{ money = 1, },
				framing_frame_3 = 	{ gold = 16, },
				mia_1 = 					{ money = 1, },
			}
		
			local ObjectInteractionManager_init_original = ObjectInteractionManager.init
			local ObjectInteractionManager_update_original = ObjectInteractionManager.update
			local ObjectInteractionManager_add_object_original = ObjectInteractionManager.add_object
			local ObjectInteractionManager_remove_object_original = ObjectInteractionManager.remove_object
		
			function ObjectInteractionManager:init(...)
				ObjectInteractionManager_init_original(self, ...)
				
				self._queued_loot_objects = {}
				self._added_loot_objects = {}
				self._total_loot_count = 0
				self._loot_count_types = {}
				for type_id, name in pairs(self.LOOT_TYPES) do
					self._loot_count_types[type_id] = 0
				end
			end
			
			function ObjectInteractionManager:update(t, ...)
				ObjectInteractionManager_update_original(self, t, ...)
				self:_check_queued_loot_objects(t)
			end
			
			function ObjectInteractionManager:add_object(obj, ...)
				table.insert(self._queued_loot_objects, { obj = obj })
				return ObjectInteractionManager_add_object_original(self, obj, ...)
			end
			
			function ObjectInteractionManager:remove_object(obj, ...)
				self:_remove_loot_object(obj)
				return ObjectInteractionManager_remove_object_original(self, obj, ...)
			end
			
			function ObjectInteractionManager:_check_queued_loot_objects(t)
				for i = #self._queued_loot_objects, 1, -1 do
					local data = self._queued_loot_objects[i]
					
					--if not data.recheck_t or data.recheck_t < t then
						local obj = data.obj
						local carry_id = obj:carry_data() and obj:carry_data():carry_id()
						local interact_type = obj:interaction() and obj:interaction().tweak_data
						
						if alive(obj) then
							local loot_type_id = carry_id and self.CARRY_ID_TO_TYPE[carry_id] or
								interact_type and self.INTERACTION_ID_TO_TYPE[interact_type]
							
							if loot_type_id then
								self:_add_loot_object(data.obj, loot_type_id)
							end
						end
						
						table.remove(self._queued_loot_objects, i)
					--end
				end
			end
			
			function ObjectInteractionManager:_add_loot_object(obj, id)
				self._added_loot_objects[obj:id()] = id
				self:_change_loot_count(1, id)
			end
			
			function ObjectInteractionManager:_remove_loot_object(obj)
				local loot_id = self._added_loot_objects[obj:id()]
				
				if loot_id then
					self:_change_loot_count(-1, loot_id)
				else
					for i, data in ipairs(self._queued_loot_objects) do
						if obj:id() == data.obj:id() then
							table.remove(self._queued_loot_objects, i)
							return
						end
					end
				end
			end
			
			function ObjectInteractionManager:_change_loot_count(change, id)
				self._total_loot_count = self._total_loot_count + change
				self._loot_count_types[id] = self._loot_count_types[id] + change
				if managers.hud and managers.hud:list_initialized() then
					local item_name = AGGREGATE_LOOT and "loot" or id
					managers.hud:get_hud_list("right_side_list"):item("loot_list"):item(item_name):change_count(change)
				end
			end
			
			function ObjectInteractionManager:loot_count(id)
				if AGGREGATE_LOOT or not id then
					return self._total_loot_count - self:_get_loot_level_compensation()
				else
					return self._loot_count_types[id] - self:_get_loot_level_compensation(id)
				end
			end
			
			function ObjectInteractionManager:_get_loot_level_compensation(id)
				local count = 0
				local level_data = managers.job and managers.job:current_level_id() and self.LEVEL_COMPENSATION[managers.job:current_level_id()]
				
				if level_data then
					if AGGREGATE_LOOT or not id then
						for _, amount in pairs(level_data) do
							count = count + amount
						end
					else
						count = level_data[id] or 0
					end
				end
				
				return count
			end
			
		end
	end
end