-------------------------------
-- Utilties Functions/Globals--
-------------------------------
-- SIMPLE MENU V2
if not SimpleMenuV2 then
	if class then 
		SimpleMenuV2 = class()

		function SimpleMenuV2:init(title, message, options)
			self.dialog_data = { title = title, text = message, button_list = {},
								 id = tostring(math.random(0,0xFFFFFFFF)) }
			self.visible = false
			for _,opt in ipairs(options) do
				local elem = {}
				elem.text = opt.text
				opt.data = opt.data or nil
				opt.callback = opt.callback or nil
				elem.callback_func = callback(self, self, "_do_callback",
											  { data = opt.data,
												callback = opt.callback,
												Class = opt.Class or nil})
				elem.cancel_button = opt.is_cancel_button or false
				if opt.is_focused_button then
					self.dialog_data.focus_button = #self.dialog_data.button_list+1
				end
				table.insert(self.dialog_data.button_list, elem)
			end
			return self
		end

		function SimpleMenuV2:_do_callback(info)
			if info.callback then
				if info.data then
					if info.Class then
						info.callback(info.Class, info.data)
					else
						info.callback(info.data)
					end
				else
					if info.Class then
						info.callback(info.Class)
					else
						info.callback()
					end
				end
			end
			self.visible = false
		end

		function SimpleMenuV2:show()
			if self.visible then
				return
			end
			self.visible = true
			managers.system_menu:show(self.dialog_data)
		end

		function SimpleMenuV2:hide()
			if self.visible then
				managers.system_menu:close(self.dialog_data.id)
				self.visible = false
				return
			end
		end
		
		patched_update_input = patched_update_input or function (self, t, dt )
			if self._data.no_buttons then
				return
			end
			
			local dir, move_time
			local move = self._controller:get_input_axis( "menu_move" )

			if( self._controller:get_input_bool( "menu_down" )) then
				dir = 1
			elseif( self._controller:get_input_bool( "menu_up" )) then
				dir = -1
			end
			
			if dir == nil then
				if move.y > self.MOVE_AXIS_LIMIT then
					dir = 1
				elseif move.y < -self.MOVE_AXIS_LIMIT then
					dir = -1
				end
			end

			if dir ~= nil then
				if( ( self._move_button_dir == dir ) and self._move_button_time and ( t < self._move_button_time + self.MOVE_AXIS_DELAY ) ) then
					move_time = self._move_button_time or t
				else
					self._panel_script:change_focus_button( dir )
					move_time = t
				end
			end

			self._move_button_dir = dir
			self._move_button_time = move_time
			
			local scroll = self._controller:get_input_axis( "menu_scroll" )
			if( scroll.y > self.MOVE_AXIS_LIMIT ) then
				self._panel_script:scroll_up()
			elseif( scroll.y < -self.MOVE_AXIS_LIMIT ) then
				self._panel_script:scroll_down()
			end
		end
		if managers then
			managers.system_menu.DIALOG_CLASS.update_input = patched_update_input
			managers.system_menu.GENERIC_DIALOG_CLASS.update_input = patched_update_input
		end
	end
end

-- Great Lua table parser/serializer :)
function Serializer (name, object, tabs)
	local function serializeKeyForTable(k)
		if type(k)=="number" then
			return "[" .. k .. "]" -- return /[1337]/	if number
		end
		if string.find(k,"[^A-z_-]") then --special symbols in it?
			return "[\"" .. k .. "\"]"
		end
		return k -- /leet/	if string
	end

	local function serializeKey(k)
		if type(k)=="number" then
			return "\t[" .. k .."] = "
		end
		if string.find(k,"[^A-z_-]") then
			return "\t[\"" .. k .. "\"] = "
		end
		return "\t" .. k .. " = "
	end

	if not tabs then tabs = "" end

	local function serialize(name, object, tabs)
		local output = tabs .. name .. " = {" .. "\n"
		
		for k,v in pairs(object) do
			local valueType = type(v)
			
			if valueType == "string" then
				output = output .. tabs .. serializeKey(k) .. string.format("%q",v)
			elseif valueType == "table" then
				output = output .. serialize(serializeKeyForTable(k), v, tabs.."\t")
			elseif valueType == "number" then
				output = output .. tabs .. serializeKey(k) .. v
			elseif valueType == "boolean" then
				output = output .. tabs .. serializeKey(k) .. tostring(v)
			else
				output = output .. tabs .. serializeKey(k) .. "\"" .. tostring(v) .. "\""
			end
			
			if next(object,k) then
				output = output .. ",\n"
			end
		end
		
		return output .. "\n" .. tabs .. "\t}"
	end
	return "function serialize_intializer()\n\tlocal " .. serialize(name, object, tabs) .. "\n\treturn " .. name .. "\nend"
end

-- SimpleInputV2
if not SimpleInputV2 then
	if managers and managers.menu_component then	
		function managers.menu_component:force_create_chat()
			self._lobby_chat_gui_active = true
			if self._game_chat_gui then
				self:show_game_chat_gui()
				return
			end
			self:add_game_chat()
		end

		SimpleInputV2 = SimpleInputV2 or class()
		SimpleInputV2._visible = SimpleInputV2._visible or false
		SimpleInputV2._origFuncs = SimpleInputV2._origFuncs or { 
															 managers_chat_send_message = managers.chat.send_message, 
															 ChatGui_create_input_panel = ChatGui._create_input_panel,
															 ChatGui_update_caret = ChatGui.update_caret,
															 ChatGui_enter_key_callback = ChatGui.enter_key_callback,
															 managers_menucomp_key_press_controller_support = managers.menu_component.key_press_controller_support,
															 managers_hudchat_on_focus = managers.hud and managers.hud._hud_chat._on_focus or nil,
															 managers_hudchat_receive_message = managers.hud and managers.hud._hud_chat.receive_message or nil
														   }
			
		function SimpleInputV2:init(opts)
			self.cblk = opts.cblk or function(...) end
			self.data = opts.data
			self.intro = opts.intro
			self.cblkself = opts.cblkself
			self.intro_color = opts.intro_color or Color.white
			self.prompt = opts.prompt
			self.isVisible = false
			self.chatWasVisible = false
		end
		
		ControllerWrapper.__get_input_pressed = ControllerWrapper.__get_input_pressed or ControllerWrapper.get_input_pressed
		function ControllerWrapper:get_input_pressed(...)
			if not SimpleInputV2._needIgnoreFirstPress then
				return self:__get_input_pressed(...)
			end
			SimpleInputV2._needIgnoreFirstPress = nil
			return false
		end
			
		function SimpleInputV2:_remove_hooks()
			managers.menu_component:close_chat_gui()
			managers.chat.send_message = SimpleInputV2._origFuncs.managers_chat_send_message
			ChatGui._create_input_panel = SimpleInputV2._origFuncs.ChatGui_create_input_panel
			ChatGui.update_caret = SimpleInputV2._origFuncs.ChatGui_update_caret
			ChatGui.enter_key_callback = SimpleInputV2._origFuncs.ChatGui_enter_key_callback
			SimpleInputV2._needIgnoreFirstPress = true
			managers.menu_component.key_press_controller_support = SimpleInputV2._origFuncs.managers_menucomp_key_press_controller_support
			if self.chatWasVisible then
				self.chatWasVisible = false
				if managers.hud then
					managers.hud._hud_chat._on_focus = SimpleInputV2._origFuncs.managers_hudchat_on_focus
					managers.hud._hud_chat.receive_message = SimpleInputV2._origFuncs.managers_hudchat_receive_message
					managers.hud:set_chat_focus( false )
				else
					managers.menu_component:_create_lobby_chat_gui()
				end
			end
		end
			
		function SimpleInputV2:_inject_hooks()
			if managers.menu_component._game_chat_gui or (managers.hud and managers.hud._hud_chat) then
				self.chatWasVisible = true
			end
			pcall(managers.menu_component.close_chat_gui, managers.menu_component)
			managers.chat.send_message = managers.chat.receive_message_by_name
			ChatGui._create_input_panel = self._create_input_panel
			ChatGui.update_caret = self._update_caret
			ChatGui.enter_key_callback = self._enter_key_callback
			managers.menu_component.key_press_controller_support = function(...) end
			managers.SimpleInputV2 = self
			managers.menu_component:force_create_chat()
			managers.menu_component._game_chat_gui.receive_message = self._receive_message
			if managers.hud then
				managers.hud._hud_chat.receive_message = function (...) end
				managers.hud._hud_chat._on_focus = function(...) end
			end
			managers.menu_component._game_chat_gui:open_page()
		end
			
		function SimpleInputV2:show()
			if SimpleInputV2._visible then
				return
			end
			SimpleInputV2._visible = true
			self.isVisible = true
			self:_inject_hooks()
			if self.intro then
				local cblkTmp = managers.SimpleInputV2.cblk
				managers.SimpleInputV2.cblk = function(...) return { msg = self.intro, color = self.intro_color } end
				managers.menu_component._game_chat_gui:receive_message(nil, nil, nil, nil)
				managers.SimpleInputV2.cblk = cblkTmp
			end
		end
			
		function SimpleInputV2:hide()
			if not SimpleInputV2._visible then
				return
			end
			SimpleInputV2._visible = false
			self._needHide = false
			self.isVisible = false
			self:_remove_hooks()
		end
			
		function SimpleInputV2:is_visible()
			return self.isVisible
		end
		
		function SimpleInputV2:_update_caret()
			if (not SimpleInputV2 or SimpleInputV2._visible) and alive(self._input_panel) then
				return SimpleInputV2._origFuncs.ChatGui_update_caret(self)
			end
		end
		
		function SimpleInputV2:_enter_key_callback()
			if not self._enabled then
				return
			end
			local text = self._input_panel:child("input_text")
			local message = text:text()
			if Idstring(message) == Idstring("/ready") then
				managers.menu_component:on_ready_pressed_mission_briefing_gui()
			elseif string.len(message) > 0 then
				managers.chat:send_message(self._channel_id, 1, message)
			else
				self._enter_loose_focus_delay = true
				self:_loose_focus()
			end
			if not SimpleInputV2 or SimpleInputV2._visible then
				self._input_panel:child("input_text"):set_text("")
				self._input_panel:child("input_text"):set_selection(0, 0)
			end
		end
			
		function SimpleInputV2:_create_input_panel()
			self._input_panel = self._panel:panel( { alpha = 0, name = "input_panel", x = 0, h = 24, w = self._panel_width, layer = 1 } )
			self._input_panel:rect( { name = "focus_indicator", visible = false, color = Color.black:with_alpha( 0.2 ), layer = 0 } )
			local say = self._input_panel:text( { name = "say", text = managers.SimpleInputV2.prompt, font = tweak_data.menu.pd2_small_font, font_size = tweak_data.menu.pd2_small_font_size, x = 0, y = 0,
											align="left", halign="left", vertical="center", hvertical="center", blend_mode="normal",
											color = Color.white, layer = 1 } )
			local _,_,w,h = say:text_rect()
			say:set_size( w, self._input_panel:h() )
			local input_text = self._input_panel:text( { name = "input_text", text = "", font = tweak_data.menu.pd2_small_font, font_size = tweak_data.menu.pd2_small_font_size, x = 0, y = 0,
											align="left", halign="left", vertical="center", hvertical="center", blend_mode="normal",
											color = Color.white, layer = 1, wrap = true, word_wrap = false } )
			local caret = self._input_panel:rect( { name="caret", layer = 2, x = 0, y = 0, w = 0, h = 0, color = Color(0.05, 1, 1, 1) } )
			self._input_panel:rect( { name="input_bg", color=Color.black:with_alpha(0.5), layer = -1, valign = "grow", h = self._input_panel:h() } )
			self._input_panel:child( "input_bg" ):set_w( self._input_panel:w() - w )
			self._input_panel:child( "input_bg" ):set_x( w )
				
			self._input_panel:stop()
			self._input_panel:animate( callback( self, self, "_animate_hide_input" ) )
		end
			
		function SimpleInputV2:_receive_message(class_test, message, ... )
			if( not alive( self._panel ) ) then
				return
			end
			if type(class_test) == "number" then
				if managers.SimpleInputV2.data then
					if managers.SimpleInputV2.cblkself then
						managers.SimpleInputV2.cblk(managers.SimpleInputV2.cblkself, message, managers.SimpleInputV2.data, ...)
					else
						managers.SimpleInputV2.cblk(message, managers.SimpleInputV2.data, ...)
					end
				end
			end
			local status, output = pcall(managers.SimpleInputV2.cblk, message, managers.SimpleInputV2.data)
			if not status or not output then
				return
			end
			local output_panel = self._panel:child( "output_panel" )
			local scroll_panel = output_panel:child( "scroll_panel" )
			local x = 0
			local icon = output.icon
			local icon_bitmap
			if icon then
				local icon_texture, icon_texture_rect = tweak_data.hud_icons:get_icon_data( icon )
				icon_bitmap = scroll_panel:bitmap( { texture = icon_texture, texture_rect = icon_texture_rect, color = output.color or Color.white, y = 1 } )
				x = icon_bitmap:right()
			end
			local line = scroll_panel:text( { text = tostring(output.msg), font = output.font or tweak_data.menu.pd2_small_font,
											font_size = output.font_size or tweak_data.menu.pd2_small_font_size, x = x, y = 0,
											align="left", halign="left", vertical="top", hvertical="top", blend_mode="normal", wrap = true, word_wrap = true,
											color = output.color or Color.white, layer = 0 } )
			local total_len = utf8.len( line:text() )                        
			line:set_range_color( 0, total_len, output.color or Color.white )
			local _,_,w,h = line:text_rect()
			line:set_h( h )
			local line_bg = scroll_panel:rect( { color=Color.black:with_alpha(0.5), layer = -1, halign="left", hvertical="top" } )
			line_bg:set_h( h )
			table.insert( self._lines, { line, line_bg, icon_bitmap } )
			self:_layout_output_panel()
			if not self._focus then
				output_panel:stop()
				output_panel:animate( callback( self, self, "_animate_show_component" ), output_panel:alpha() )
				output_panel:animate( callback( self, self, "_animate_fade_output" ) )
			end
		end
	end
end